##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`app.bsky.graph.getActorStarterPacks`."""

    actor: string_formats.AtIdentifier  #: Actor.
    cursor: t.Optional[str] = None  #: Cursor.
    limit: t.Optional[int] = Field(default=50, ge=1, le=100)  #: Limit.


class ParamsDict(t.TypedDict):
    actor: string_formats.AtIdentifier  #: Actor.
    cursor: te.NotRequired[t.Optional[str]]  #: Cursor.
    limit: te.NotRequired[t.Optional[int]]  #: Limit.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`app.bsky.graph.getActorStarterPacks`."""

    starter_packs: t.List['models.AppBskyGraphDefs.StarterPackViewBasic']  #: Starter packs.
    cursor: t.Optional[str] = None  #: Cursor.



from atproto_core.exceptions import AtProtocolError


class InvalidCompressedPubkeyError(AtProtocolError): ...


class DidKeyError(AtProtocolError): ...


class IncorrectMultikeyPrefixError(DidKeyError): ...


class IncorrectDidKeyPrefixError(DidKeyError): ...


class UnsupportedKeyTypeError(DidKeyError): ...


class UnsupportedSignatureAlgorithmError(AtProtocolError): ...



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

from atproto_client.models import base, string_formats


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`com.atproto.sync.listBlobs`."""

    did: string_formats.Did  #: The DID of the repo.
    cursor: t.Optional[str] = None  #: Cursor.
    limit: t.Optional[int] = Field(default=500, ge=1, le=1000)  #: Limit.
    since: t.Optional[string_formats.Tid] = None  #: Optional revision of the repo to list blobs since.


class ParamsDict(t.TypedDict):
    did: string_formats.Did  #: The DID of the repo.
    cursor: te.NotRequired[t.Optional[str]]  #: Cursor.
    limit: te.NotRequired[t.Optional[int]]  #: Limit.
    since: te.NotRequired[t.Optional[string_formats.Tid]]  #: Optional revision of the repo to list blobs since.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.sync.listBlobs`."""

    cids: t.List[string_formats.Cid]  #: Cids.
    cursor: t.Optional[str] = None  #: Cursor.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`app.bsky.graph.getSuggestedFollowsByActor`."""

    actor: string_formats.AtIdentifier  #: Actor.


class ParamsDict(t.TypedDict):
    actor: string_formats.AtIdentifier  #: Actor.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`app.bsky.graph.getSuggestedFollowsByActor`."""

    suggestions: t.List['models.AppBskyActorDefs.ProfileView']  #: Suggestions.
    is_fallback: t.Optional[bool] = (
        False  #: If true, response has fallen-back to generic results, and is not scoped using relativeToDid.
    )
    rec_id: t.Optional[int] = None  #: Snowflake for this recommendation, use when submitting recommendation events.



import re
import subprocess
import typing as t
from pathlib import Path

from atproto_core.exceptions import InvalidNsidError
from atproto_core.nsid import NSID


def format_code(filepath: Path, quiet: bool = True) -> None:
    if not isinstance(filepath, Path):
        return

    quiet_option = '--quiet'
    if not quiet:
        quiet_option = ''

    # FIXME(MarshalX): doesn't work well with not-project dir
    subprocess.run(['ruff', 'format', quiet_option, filepath])  # noqa: S603, S607
    subprocess.run(['ruff', 'check', quiet_option, '--fix', filepath])  # noqa: S603, S607
    subprocess.run(['ruff', 'format', quiet_option, filepath])  # noqa: S603, S607


def append_code(filepath: Path, code: str) -> None:
    _write_code(filepath, code, append=True)


def write_code(filepath: Path, code: str) -> None:
    _write_code(filepath, code)


def _write_code(filepath: Path, code: str, *, append: bool = False) -> None:
    filepath.parent.mkdir(exist_ok=True, parents=True)

    mode = 'w'
    if append:
        mode = 'a'

    with open(filepath, mode=mode, encoding='UTF-8') as f:
        f.write(code)


def get_file_path_parts(nsid: NSID) -> t.List[str]:
    return nsid.segments[:-1] + [f'{convert_camel_case_to_snake_case(nsid.name)}.py']


def get_import_path_old(nsid: NSID) -> str:
    return '.'.join(nsid.segments[:-1] + [f'{convert_camel_case_to_snake_case(nsid.name)}'])


def get_import_path(nsid: NSID) -> str:
    nsid_parts = nsid.segments[:-1] + camel_case_split(nsid.name)
    return ''.join([p.capitalize() for p in nsid_parts])


def convert_camel_case_to_snake_case(string: str) -> str:
    s = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', string)
    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s).lower()


def camel_case_split(string: str) -> t.List[str]:
    # regex by chatgpt
    return re.findall(r'[A-Z]?[a-z]+|[A-Z]+(?=[A-Z]|$)', string)


def gen_description_by_camel_case_name(name: str) -> str:
    words = camel_case_split(name)
    words = [w.lower() for w in words]
    words[0] = words[0].capitalize()
    return ' '.join(words)


def sort_dict_by_key(d: dict) -> dict:
    return dict(sorted(d.items()))


def get_code_intent(level: int) -> str:
    return ' ' * 4 * level


def join_code(lines: t.List[str]) -> str:
    return '\n'.join(lines)


def get_sync_async_keywords(*, sync: bool) -> t.Tuple[str, str]:
    definition, call = 'async ', 'await '
    if sync:
        definition, call = '', ''

    return definition, call


def capitalize_first_symbol(string: str) -> str:
    if string and string[0].islower():
        chars = list(string[1:])
        chars.insert(0, string[0].upper())
        return ''.join(chars)

    return string


def get_def_model_name(method_name: str) -> str:
    return f'{capitalize_first_symbol(method_name)}'


def get_record_model_name(_: str) -> str:
    return 'Record'


def get_model_path(nsid: NSID, method_name: str) -> str:
    return f'models.{get_import_path(nsid)}.{get_def_model_name(method_name)}'


def _resolve_nsid_ref(nsid: NSID, ref: str, *, local: bool = False) -> t.Tuple[str, str]:
    """Return the path to the model and model name."""
    if '#' in ref:
        ref_nsid_str, def_name = ref.split('#', 1)
        def_name = get_def_model_name(def_name)

        try:
            ref_nsid = NSID.from_str(ref_nsid_str)
            return get_model_path(ref_nsid, def_name), def_name
        except InvalidNsidError:
            if local:
                return def_name, def_name
            return get_model_path(nsid, def_name), def_name
    else:
        ref_nsid = NSID.from_str(ref)
        def_name = get_def_model_name(nsid.name)

        if local:
            return def_name, def_name

        # FIXME(MarshalX): Is it works well? ;d
        return get_model_path(ref_nsid, 'Main'), def_name



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te

from atproto_client.models import base


class Data(base.DataModelBase):
    """Input data model for :obj:`com.atproto.admin.disableInviteCodes`."""

    accounts: t.Optional[t.List[str]] = None  #: Accounts.
    codes: t.Optional[t.List[str]] = None  #: Codes.


class DataDict(t.TypedDict):
    accounts: te.NotRequired[t.Optional[t.List[str]]]  #: Accounts.
    codes: te.NotRequired[t.Optional[t.List[str]]]  #: Codes.



import os
import typing as t
import warnings

from pydantic import BaseModel, ConfigDict, Field, alias_generators, model_validator

from atproto_client.exceptions import ModelFieldNotFoundError


def _alias_generator(name: str) -> str:
    camel_name = alias_generators.to_camel(name)

    if camel_name.endswith('_'):
        camel_name = camel_name[:-1]

    return camel_name


class AtProtocolBase: ...


class ModelBase(BaseModel, AtProtocolBase):
    """Base class for all data classes.

    Provides square brackets [] notation to get attributes like in a dictionary.
    """

    model_config = ConfigDict(extra='allow', alias_generator=_alias_generator, populate_by_name=True, strict=True)

    def __getitem__(self, item: str) -> t.Any:
        if hasattr(self, item):
            return getattr(self, item)

        raise ModelFieldNotFoundError(f"Can't find field '{item}' in the object of type {type(self)}.")

    @model_validator(mode='after')
    def __alert_about_extra_fields(self) -> 'ModelBase':
        # used for debugging purposes because gives false positives due to the way pydantic works

        if self.model_extra and os.environ.get('ATPROTO_LEXICON_WARN', '0') == '1':
            warnings.warn(
                f'Extra fields found in the object of type {type(self)}: {self.model_extra}. '  # noqa: S608
                f'Probably you are using the old version of SDK. '
                f'Please update it using `pip install -U atproto`. '
                f'In case you are working with custom lexicon ignore this warning. '
                f'It is also possible that you are working with extended record. '
                f'To disable this warning set `ATPROTO_LEXICON_WARN` to `0` in the environment variables.',
                stacklevel=0,
            )

        return self


class ParamsModelBase(ModelBase):
    pass


class DataModelBase(ModelBase):
    pass


class ResponseModelBase(ModelBase):
    pass


class SugarResponseModelBase(ResponseModelBase):
    pass


class UnknownDict(AtProtocolBase):
    pass


class UnknownRecord(ModelBase):
    pass


class UnknownUnionModel(ModelBase):
    py_type: str = Field(alias='$type')


class RecordModelBase(UnknownRecord):
    pass



from atproto_client.models.app.bsky.actor import defs as AppBskyActorDefs
from atproto_client.models.app.bsky.actor import get_preferences as AppBskyActorGetPreferences
from atproto_client.models.app.bsky.actor import get_profile as AppBskyActorGetProfile
from atproto_client.models.app.bsky.actor import get_profiles as AppBskyActorGetProfiles
from atproto_client.models.app.bsky.actor import get_suggestions as AppBskyActorGetSuggestions
from atproto_client.models.app.bsky.actor import profile as AppBskyActorProfile
from atproto_client.models.app.bsky.actor import put_preferences as AppBskyActorPutPreferences
from atproto_client.models.app.bsky.actor import search_actors as AppBskyActorSearchActors
from atproto_client.models.app.bsky.actor import search_actors_typeahead as AppBskyActorSearchActorsTypeahead
from atproto_client.models.app.bsky.embed import defs as AppBskyEmbedDefs
from atproto_client.models.app.bsky.embed import external as AppBskyEmbedExternal
from atproto_client.models.app.bsky.embed import images as AppBskyEmbedImages
from atproto_client.models.app.bsky.embed import record as AppBskyEmbedRecord
from atproto_client.models.app.bsky.embed import record_with_media as AppBskyEmbedRecordWithMedia
from atproto_client.models.app.bsky.embed import video as AppBskyEmbedVideo
from atproto_client.models.app.bsky.feed import defs as AppBskyFeedDefs
from atproto_client.models.app.bsky.feed import describe_feed_generator as AppBskyFeedDescribeFeedGenerator
from atproto_client.models.app.bsky.feed import generator as AppBskyFeedGenerator
from atproto_client.models.app.bsky.feed import get_actor_feeds as AppBskyFeedGetActorFeeds
from atproto_client.models.app.bsky.feed import get_actor_likes as AppBskyFeedGetActorLikes
from atproto_client.models.app.bsky.feed import get_author_feed as AppBskyFeedGetAuthorFeed
from atproto_client.models.app.bsky.feed import get_feed as AppBskyFeedGetFeed
from atproto_client.models.app.bsky.feed import get_feed_generator as AppBskyFeedGetFeedGenerator
from atproto_client.models.app.bsky.feed import get_feed_generators as AppBskyFeedGetFeedGenerators
from atproto_client.models.app.bsky.feed import get_feed_skeleton as AppBskyFeedGetFeedSkeleton
from atproto_client.models.app.bsky.feed import get_likes as AppBskyFeedGetLikes
from atproto_client.models.app.bsky.feed import get_list_feed as AppBskyFeedGetListFeed
from atproto_client.models.app.bsky.feed import get_post_thread as AppBskyFeedGetPostThread
from atproto_client.models.app.bsky.feed import get_posts as AppBskyFeedGetPosts
from atproto_client.models.app.bsky.feed import get_quotes as AppBskyFeedGetQuotes
from atproto_client.models.app.bsky.feed import get_reposted_by as AppBskyFeedGetRepostedBy
from atproto_client.models.app.bsky.feed import get_suggested_feeds as AppBskyFeedGetSuggestedFeeds
from atproto_client.models.app.bsky.feed import get_timeline as AppBskyFeedGetTimeline
from atproto_client.models.app.bsky.feed import like as AppBskyFeedLike
from atproto_client.models.app.bsky.feed import post as AppBskyFeedPost
from atproto_client.models.app.bsky.feed import postgate as AppBskyFeedPostgate
from atproto_client.models.app.bsky.feed import repost as AppBskyFeedRepost
from atproto_client.models.app.bsky.feed import search_posts as AppBskyFeedSearchPosts
from atproto_client.models.app.bsky.feed import send_interactions as AppBskyFeedSendInteractions
from atproto_client.models.app.bsky.feed import threadgate as AppBskyFeedThreadgate
from atproto_client.models.app.bsky.graph import block as AppBskyGraphBlock
from atproto_client.models.app.bsky.graph import defs as AppBskyGraphDefs
from atproto_client.models.app.bsky.graph import follow as AppBskyGraphFollow
from atproto_client.models.app.bsky.graph import get_actor_starter_packs as AppBskyGraphGetActorStarterPacks
from atproto_client.models.app.bsky.graph import get_blocks as AppBskyGraphGetBlocks
from atproto_client.models.app.bsky.graph import get_followers as AppBskyGraphGetFollowers
from atproto_client.models.app.bsky.graph import get_follows as AppBskyGraphGetFollows
from atproto_client.models.app.bsky.graph import get_known_followers as AppBskyGraphGetKnownFollowers
from atproto_client.models.app.bsky.graph import get_list as AppBskyGraphGetList
from atproto_client.models.app.bsky.graph import get_list_blocks as AppBskyGraphGetListBlocks
from atproto_client.models.app.bsky.graph import get_list_mutes as AppBskyGraphGetListMutes
from atproto_client.models.app.bsky.graph import get_lists as AppBskyGraphGetLists
from atproto_client.models.app.bsky.graph import get_mutes as AppBskyGraphGetMutes
from atproto_client.models.app.bsky.graph import get_relationships as AppBskyGraphGetRelationships
from atproto_client.models.app.bsky.graph import get_starter_pack as AppBskyGraphGetStarterPack
from atproto_client.models.app.bsky.graph import get_starter_packs as AppBskyGraphGetStarterPacks
from atproto_client.models.app.bsky.graph import (
    get_suggested_follows_by_actor as AppBskyGraphGetSuggestedFollowsByActor,
)
from atproto_client.models.app.bsky.graph import list as AppBskyGraphList
from atproto_client.models.app.bsky.graph import listblock as AppBskyGraphListblock
from atproto_client.models.app.bsky.graph import listitem as AppBskyGraphListitem
from atproto_client.models.app.bsky.graph import mute_actor as AppBskyGraphMuteActor
from atproto_client.models.app.bsky.graph import mute_actor_list as AppBskyGraphMuteActorList
from atproto_client.models.app.bsky.graph import mute_thread as AppBskyGraphMuteThread
from atproto_client.models.app.bsky.graph import search_starter_packs as AppBskyGraphSearchStarterPacks
from atproto_client.models.app.bsky.graph import starterpack as AppBskyGraphStarterpack
from atproto_client.models.app.bsky.graph import unmute_actor as AppBskyGraphUnmuteActor
from atproto_client.models.app.bsky.graph import unmute_actor_list as AppBskyGraphUnmuteActorList
from atproto_client.models.app.bsky.graph import unmute_thread as AppBskyGraphUnmuteThread
from atproto_client.models.app.bsky.graph import verification as AppBskyGraphVerification
from atproto_client.models.app.bsky.labeler import defs as AppBskyLabelerDefs
from atproto_client.models.app.bsky.labeler import get_services as AppBskyLabelerGetServices
from atproto_client.models.app.bsky.labeler import service as AppBskyLabelerService
from atproto_client.models.app.bsky.notification import defs as AppBskyNotificationDefs
from atproto_client.models.app.bsky.notification import get_unread_count as AppBskyNotificationGetUnreadCount
from atproto_client.models.app.bsky.notification import list_notifications as AppBskyNotificationListNotifications
from atproto_client.models.app.bsky.notification import put_preferences as AppBskyNotificationPutPreferences
from atproto_client.models.app.bsky.notification import register_push as AppBskyNotificationRegisterPush
from atproto_client.models.app.bsky.notification import update_seen as AppBskyNotificationUpdateSeen
from atproto_client.models.app.bsky.richtext import facet as AppBskyRichtextFacet
from atproto_client.models.app.bsky.unspecced import defs as AppBskyUnspeccedDefs
from atproto_client.models.app.bsky.unspecced import get_config as AppBskyUnspeccedGetConfig
from atproto_client.models.app.bsky.unspecced import (
    get_popular_feed_generators as AppBskyUnspeccedGetPopularFeedGenerators,
)
from atproto_client.models.app.bsky.unspecced import get_suggested_feeds as AppBskyUnspeccedGetSuggestedFeeds
from atproto_client.models.app.bsky.unspecced import (
    get_suggested_feeds_skeleton as AppBskyUnspeccedGetSuggestedFeedsSkeleton,
)
from atproto_client.models.app.bsky.unspecced import (
    get_suggested_starter_packs as AppBskyUnspeccedGetSuggestedStarterPacks,
)
from atproto_client.models.app.bsky.unspecced import (
    get_suggested_starter_packs_skeleton as AppBskyUnspeccedGetSuggestedStarterPacksSkeleton,
)
from atproto_client.models.app.bsky.unspecced import get_suggested_users as AppBskyUnspeccedGetSuggestedUsers
from atproto_client.models.app.bsky.unspecced import (
    get_suggested_users_skeleton as AppBskyUnspeccedGetSuggestedUsersSkeleton,
)
from atproto_client.models.app.bsky.unspecced import get_suggestions_skeleton as AppBskyUnspeccedGetSuggestionsSkeleton
from atproto_client.models.app.bsky.unspecced import get_tagged_suggestions as AppBskyUnspeccedGetTaggedSuggestions
from atproto_client.models.app.bsky.unspecced import get_trending_topics as AppBskyUnspeccedGetTrendingTopics
from atproto_client.models.app.bsky.unspecced import get_trends as AppBskyUnspeccedGetTrends
from atproto_client.models.app.bsky.unspecced import get_trends_skeleton as AppBskyUnspeccedGetTrendsSkeleton
from atproto_client.models.app.bsky.unspecced import search_actors_skeleton as AppBskyUnspeccedSearchActorsSkeleton
from atproto_client.models.app.bsky.unspecced import search_posts_skeleton as AppBskyUnspeccedSearchPostsSkeleton
from atproto_client.models.app.bsky.unspecced import (
    search_starter_packs_skeleton as AppBskyUnspeccedSearchStarterPacksSkeleton,
)
from atproto_client.models.app.bsky.video import defs as AppBskyVideoDefs
from atproto_client.models.app.bsky.video import get_job_status as AppBskyVideoGetJobStatus
from atproto_client.models.app.bsky.video import get_upload_limits as AppBskyVideoGetUploadLimits
from atproto_client.models.app.bsky.video import upload_video as AppBskyVideoUploadVideo
from atproto_client.models.chat.bsky.actor import declaration as ChatBskyActorDeclaration
from atproto_client.models.chat.bsky.actor import defs as ChatBskyActorDefs
from atproto_client.models.chat.bsky.actor import delete_account as ChatBskyActorDeleteAccount
from atproto_client.models.chat.bsky.actor import export_account_data as ChatBskyActorExportAccountData
from atproto_client.models.chat.bsky.convo import accept_convo as ChatBskyConvoAcceptConvo
from atproto_client.models.chat.bsky.convo import add_reaction as ChatBskyConvoAddReaction
from atproto_client.models.chat.bsky.convo import defs as ChatBskyConvoDefs
from atproto_client.models.chat.bsky.convo import delete_message_for_self as ChatBskyConvoDeleteMessageForSelf
from atproto_client.models.chat.bsky.convo import get_convo as ChatBskyConvoGetConvo
from atproto_client.models.chat.bsky.convo import get_convo_availability as ChatBskyConvoGetConvoAvailability
from atproto_client.models.chat.bsky.convo import get_convo_for_members as ChatBskyConvoGetConvoForMembers
from atproto_client.models.chat.bsky.convo import get_log as ChatBskyConvoGetLog
from atproto_client.models.chat.bsky.convo import get_messages as ChatBskyConvoGetMessages
from atproto_client.models.chat.bsky.convo import leave_convo as ChatBskyConvoLeaveConvo
from atproto_client.models.chat.bsky.convo import list_convos as ChatBskyConvoListConvos
from atproto_client.models.chat.bsky.convo import mute_convo as ChatBskyConvoMuteConvo
from atproto_client.models.chat.bsky.convo import remove_reaction as ChatBskyConvoRemoveReaction
from atproto_client.models.chat.bsky.convo import send_message as ChatBskyConvoSendMessage
from atproto_client.models.chat.bsky.convo import send_message_batch as ChatBskyConvoSendMessageBatch
from atproto_client.models.chat.bsky.convo import unmute_convo as ChatBskyConvoUnmuteConvo
from atproto_client.models.chat.bsky.convo import update_all_read as ChatBskyConvoUpdateAllRead
from atproto_client.models.chat.bsky.convo import update_read as ChatBskyConvoUpdateRead
from atproto_client.models.chat.bsky.moderation import get_actor_metadata as ChatBskyModerationGetActorMetadata
from atproto_client.models.chat.bsky.moderation import get_message_context as ChatBskyModerationGetMessageContext
from atproto_client.models.chat.bsky.moderation import update_actor_access as ChatBskyModerationUpdateActorAccess
from atproto_client.models.com.atproto.admin import defs as ComAtprotoAdminDefs
from atproto_client.models.com.atproto.admin import delete_account as ComAtprotoAdminDeleteAccount
from atproto_client.models.com.atproto.admin import disable_account_invites as ComAtprotoAdminDisableAccountInvites
from atproto_client.models.com.atproto.admin import disable_invite_codes as ComAtprotoAdminDisableInviteCodes
from atproto_client.models.com.atproto.admin import enable_account_invites as ComAtprotoAdminEnableAccountInvites
from atproto_client.models.com.atproto.admin import get_account_info as ComAtprotoAdminGetAccountInfo
from atproto_client.models.com.atproto.admin import get_account_infos as ComAtprotoAdminGetAccountInfos
from atproto_client.models.com.atproto.admin import get_invite_codes as ComAtprotoAdminGetInviteCodes
from atproto_client.models.com.atproto.admin import get_subject_status as ComAtprotoAdminGetSubjectStatus
from atproto_client.models.com.atproto.admin import search_accounts as ComAtprotoAdminSearchAccounts
from atproto_client.models.com.atproto.admin import send_email as ComAtprotoAdminSendEmail
from atproto_client.models.com.atproto.admin import update_account_email as ComAtprotoAdminUpdateAccountEmail
from atproto_client.models.com.atproto.admin import update_account_handle as ComAtprotoAdminUpdateAccountHandle
from atproto_client.models.com.atproto.admin import update_account_password as ComAtprotoAdminUpdateAccountPassword
from atproto_client.models.com.atproto.admin import update_account_signing_key as ComAtprotoAdminUpdateAccountSigningKey
from atproto_client.models.com.atproto.admin import update_subject_status as ComAtprotoAdminUpdateSubjectStatus
from atproto_client.models.com.atproto.identity import defs as ComAtprotoIdentityDefs
from atproto_client.models.com.atproto.identity import (
    get_recommended_did_credentials as ComAtprotoIdentityGetRecommendedDidCredentials,
)
from atproto_client.models.com.atproto.identity import refresh_identity as ComAtprotoIdentityRefreshIdentity
from atproto_client.models.com.atproto.identity import (
    request_plc_operation_signature as ComAtprotoIdentityRequestPlcOperationSignature,
)
from atproto_client.models.com.atproto.identity import resolve_did as ComAtprotoIdentityResolveDid
from atproto_client.models.com.atproto.identity import resolve_handle as ComAtprotoIdentityResolveHandle
from atproto_client.models.com.atproto.identity import resolve_identity as ComAtprotoIdentityResolveIdentity
from atproto_client.models.com.atproto.identity import sign_plc_operation as ComAtprotoIdentitySignPlcOperation
from atproto_client.models.com.atproto.identity import submit_plc_operation as ComAtprotoIdentitySubmitPlcOperation
from atproto_client.models.com.atproto.identity import update_handle as ComAtprotoIdentityUpdateHandle
from atproto_client.models.com.atproto.label import defs as ComAtprotoLabelDefs
from atproto_client.models.com.atproto.label import query_labels as ComAtprotoLabelQueryLabels
from atproto_client.models.com.atproto.label import subscribe_labels as ComAtprotoLabelSubscribeLabels
from atproto_client.models.com.atproto.lexicon import schema as ComAtprotoLexiconSchema
from atproto_client.models.com.atproto.moderation import create_report as ComAtprotoModerationCreateReport
from atproto_client.models.com.atproto.moderation import defs as ComAtprotoModerationDefs
from atproto_client.models.com.atproto.repo import apply_writes as ComAtprotoRepoApplyWrites
from atproto_client.models.com.atproto.repo import create_record as ComAtprotoRepoCreateRecord
from atproto_client.models.com.atproto.repo import defs as ComAtprotoRepoDefs
from atproto_client.models.com.atproto.repo import delete_record as ComAtprotoRepoDeleteRecord
from atproto_client.models.com.atproto.repo import describe_repo as ComAtprotoRepoDescribeRepo
from atproto_client.models.com.atproto.repo import get_record as ComAtprotoRepoGetRecord
from atproto_client.models.com.atproto.repo import import_repo as ComAtprotoRepoImportRepo
from atproto_client.models.com.atproto.repo import list_missing_blobs as ComAtprotoRepoListMissingBlobs
from atproto_client.models.com.atproto.repo import list_records as ComAtprotoRepoListRecords
from atproto_client.models.com.atproto.repo import put_record as ComAtprotoRepoPutRecord
from atproto_client.models.com.atproto.repo import strong_ref as ComAtprotoRepoStrongRef
from atproto_client.models.com.atproto.repo import upload_blob as ComAtprotoRepoUploadBlob
from atproto_client.models.com.atproto.server import activate_account as ComAtprotoServerActivateAccount
from atproto_client.models.com.atproto.server import check_account_status as ComAtprotoServerCheckAccountStatus
from atproto_client.models.com.atproto.server import confirm_email as ComAtprotoServerConfirmEmail
from atproto_client.models.com.atproto.server import create_account as ComAtprotoServerCreateAccount
from atproto_client.models.com.atproto.server import create_app_password as ComAtprotoServerCreateAppPassword
from atproto_client.models.com.atproto.server import create_invite_code as ComAtprotoServerCreateInviteCode
from atproto_client.models.com.atproto.server import create_invite_codes as ComAtprotoServerCreateInviteCodes
from atproto_client.models.com.atproto.server import create_session as ComAtprotoServerCreateSession
from atproto_client.models.com.atproto.server import deactivate_account as ComAtprotoServerDeactivateAccount
from atproto_client.models.com.atproto.server import defs as ComAtprotoServerDefs
from atproto_client.models.com.atproto.server import delete_account as ComAtprotoServerDeleteAccount
from atproto_client.models.com.atproto.server import delete_session as ComAtprotoServerDeleteSession
from atproto_client.models.com.atproto.server import describe_server as ComAtprotoServerDescribeServer
from atproto_client.models.com.atproto.server import get_account_invite_codes as ComAtprotoServerGetAccountInviteCodes
from atproto_client.models.com.atproto.server import get_service_auth as ComAtprotoServerGetServiceAuth
from atproto_client.models.com.atproto.server import get_session as ComAtprotoServerGetSession
from atproto_client.models.com.atproto.server import list_app_passwords as ComAtprotoServerListAppPasswords
from atproto_client.models.com.atproto.server import refresh_session as ComAtprotoServerRefreshSession
from atproto_client.models.com.atproto.server import request_account_delete as ComAtprotoServerRequestAccountDelete
from atproto_client.models.com.atproto.server import (
    request_email_confirmation as ComAtprotoServerRequestEmailConfirmation,
)
from atproto_client.models.com.atproto.server import request_email_update as ComAtprotoServerRequestEmailUpdate
from atproto_client.models.com.atproto.server import request_password_reset as ComAtprotoServerRequestPasswordReset
from atproto_client.models.com.atproto.server import reserve_signing_key as ComAtprotoServerReserveSigningKey
from atproto_client.models.com.atproto.server import reset_password as ComAtprotoServerResetPassword
from atproto_client.models.com.atproto.server import revoke_app_password as ComAtprotoServerRevokeAppPassword
from atproto_client.models.com.atproto.server import update_email as ComAtprotoServerUpdateEmail
from atproto_client.models.com.atproto.sync import defs as ComAtprotoSyncDefs
from atproto_client.models.com.atproto.sync import get_blob as ComAtprotoSyncGetBlob
from atproto_client.models.com.atproto.sync import get_blocks as ComAtprotoSyncGetBlocks
from atproto_client.models.com.atproto.sync import get_checkout as ComAtprotoSyncGetCheckout
from atproto_client.models.com.atproto.sync import get_head as ComAtprotoSyncGetHead
from atproto_client.models.com.atproto.sync import get_host_status as ComAtprotoSyncGetHostStatus
from atproto_client.models.com.atproto.sync import get_latest_commit as ComAtprotoSyncGetLatestCommit
from atproto_client.models.com.atproto.sync import get_record as ComAtprotoSyncGetRecord
from atproto_client.models.com.atproto.sync import get_repo as ComAtprotoSyncGetRepo
from atproto_client.models.com.atproto.sync import get_repo_status as ComAtprotoSyncGetRepoStatus
from atproto_client.models.com.atproto.sync import list_blobs as ComAtprotoSyncListBlobs
from atproto_client.models.com.atproto.sync import list_hosts as ComAtprotoSyncListHosts
from atproto_client.models.com.atproto.sync import list_repos as ComAtprotoSyncListRepos
from atproto_client.models.com.atproto.sync import list_repos_by_collection as ComAtprotoSyncListReposByCollection
from atproto_client.models.com.atproto.sync import notify_of_update as ComAtprotoSyncNotifyOfUpdate
from atproto_client.models.com.atproto.sync import request_crawl as ComAtprotoSyncRequestCrawl
from atproto_client.models.com.atproto.sync import subscribe_repos as ComAtprotoSyncSubscribeRepos
from atproto_client.models.com.atproto.temp import add_reserved_handle as ComAtprotoTempAddReservedHandle
from atproto_client.models.com.atproto.temp import check_signup_queue as ComAtprotoTempCheckSignupQueue
from atproto_client.models.com.atproto.temp import fetch_labels as ComAtprotoTempFetchLabels
from atproto_client.models.com.atproto.temp import request_phone_verification as ComAtprotoTempRequestPhoneVerification
from atproto_client.models.models_loader import load_models
from atproto_client.models.tools.ozone.communication import create_template as ToolsOzoneCommunicationCreateTemplate
from atproto_client.models.tools.ozone.communication import defs as ToolsOzoneCommunicationDefs
from atproto_client.models.tools.ozone.communication import delete_template as ToolsOzoneCommunicationDeleteTemplate
from atproto_client.models.tools.ozone.communication import list_templates as ToolsOzoneCommunicationListTemplates
from atproto_client.models.tools.ozone.communication import update_template as ToolsOzoneCommunicationUpdateTemplate
from atproto_client.models.tools.ozone.moderation import defs as ToolsOzoneModerationDefs
from atproto_client.models.tools.ozone.moderation import emit_event as ToolsOzoneModerationEmitEvent
from atproto_client.models.tools.ozone.moderation import get_event as ToolsOzoneModerationGetEvent
from atproto_client.models.tools.ozone.moderation import get_record as ToolsOzoneModerationGetRecord
from atproto_client.models.tools.ozone.moderation import get_records as ToolsOzoneModerationGetRecords
from atproto_client.models.tools.ozone.moderation import get_repo as ToolsOzoneModerationGetRepo
from atproto_client.models.tools.ozone.moderation import get_reporter_stats as ToolsOzoneModerationGetReporterStats
from atproto_client.models.tools.ozone.moderation import get_repos as ToolsOzoneModerationGetRepos
from atproto_client.models.tools.ozone.moderation import get_subjects as ToolsOzoneModerationGetSubjects
from atproto_client.models.tools.ozone.moderation import query_events as ToolsOzoneModerationQueryEvents
from atproto_client.models.tools.ozone.moderation import query_statuses as ToolsOzoneModerationQueryStatuses
from atproto_client.models.tools.ozone.moderation import search_repos as ToolsOzoneModerationSearchRepos
from atproto_client.models.tools.ozone.server import get_config as ToolsOzoneServerGetConfig
from atproto_client.models.tools.ozone.set import add_values as ToolsOzoneSetAddValues
from atproto_client.models.tools.ozone.set import defs as ToolsOzoneSetDefs
from atproto_client.models.tools.ozone.set import delete_set as ToolsOzoneSetDeleteSet
from atproto_client.models.tools.ozone.set import delete_values as ToolsOzoneSetDeleteValues
from atproto_client.models.tools.ozone.set import get_values as ToolsOzoneSetGetValues
from atproto_client.models.tools.ozone.set import query_sets as ToolsOzoneSetQuerySets
from atproto_client.models.tools.ozone.set import upsert_set as ToolsOzoneSetUpsertSet
from atproto_client.models.tools.ozone.setting import defs as ToolsOzoneSettingDefs
from atproto_client.models.tools.ozone.setting import list_options as ToolsOzoneSettingListOptions
from atproto_client.models.tools.ozone.setting import remove_options as ToolsOzoneSettingRemoveOptions
from atproto_client.models.tools.ozone.setting import upsert_option as ToolsOzoneSettingUpsertOption
from atproto_client.models.tools.ozone.signature import defs as ToolsOzoneSignatureDefs
from atproto_client.models.tools.ozone.signature import find_correlation as ToolsOzoneSignatureFindCorrelation
from atproto_client.models.tools.ozone.signature import find_related_accounts as ToolsOzoneSignatureFindRelatedAccounts
from atproto_client.models.tools.ozone.signature import search_accounts as ToolsOzoneSignatureSearchAccounts
from atproto_client.models.tools.ozone.team import add_member as ToolsOzoneTeamAddMember
from atproto_client.models.tools.ozone.team import defs as ToolsOzoneTeamDefs
from atproto_client.models.tools.ozone.team import delete_member as ToolsOzoneTeamDeleteMember
from atproto_client.models.tools.ozone.team import list_members as ToolsOzoneTeamListMembers
from atproto_client.models.tools.ozone.team import update_member as ToolsOzoneTeamUpdateMember
from atproto_client.models.utils import (
    create_strong_ref,
    get_model_as_dict,
    get_model_as_json,
    get_or_create,
    is_record_type,
)


class _Ids:
    AppBskyActorDefs: str = 'app.bsky.actor.defs'
    AppBskyActorGetPreferences: str = 'app.bsky.actor.getPreferences'
    AppBskyActorGetProfile: str = 'app.bsky.actor.getProfile'
    AppBskyActorGetProfiles: str = 'app.bsky.actor.getProfiles'
    AppBskyActorGetSuggestions: str = 'app.bsky.actor.getSuggestions'
    AppBskyActorProfile: str = 'app.bsky.actor.profile'
    AppBskyActorPutPreferences: str = 'app.bsky.actor.putPreferences'
    AppBskyActorSearchActors: str = 'app.bsky.actor.searchActors'
    AppBskyActorSearchActorsTypeahead: str = 'app.bsky.actor.searchActorsTypeahead'
    AppBskyEmbedDefs: str = 'app.bsky.embed.defs'
    AppBskyEmbedExternal: str = 'app.bsky.embed.external'
    AppBskyEmbedImages: str = 'app.bsky.embed.images'
    AppBskyEmbedRecord: str = 'app.bsky.embed.record'
    AppBskyEmbedRecordWithMedia: str = 'app.bsky.embed.recordWithMedia'
    AppBskyEmbedVideo: str = 'app.bsky.embed.video'
    AppBskyFeedDefs: str = 'app.bsky.feed.defs'
    AppBskyFeedDescribeFeedGenerator: str = 'app.bsky.feed.describeFeedGenerator'
    AppBskyFeedGenerator: str = 'app.bsky.feed.generator'
    AppBskyFeedGetActorFeeds: str = 'app.bsky.feed.getActorFeeds'
    AppBskyFeedGetActorLikes: str = 'app.bsky.feed.getActorLikes'
    AppBskyFeedGetAuthorFeed: str = 'app.bsky.feed.getAuthorFeed'
    AppBskyFeedGetFeed: str = 'app.bsky.feed.getFeed'
    AppBskyFeedGetFeedGenerator: str = 'app.bsky.feed.getFeedGenerator'
    AppBskyFeedGetFeedGenerators: str = 'app.bsky.feed.getFeedGenerators'
    AppBskyFeedGetFeedSkeleton: str = 'app.bsky.feed.getFeedSkeleton'
    AppBskyFeedGetLikes: str = 'app.bsky.feed.getLikes'
    AppBskyFeedGetListFeed: str = 'app.bsky.feed.getListFeed'
    AppBskyFeedGetPostThread: str = 'app.bsky.feed.getPostThread'
    AppBskyFeedGetPosts: str = 'app.bsky.feed.getPosts'
    AppBskyFeedGetQuotes: str = 'app.bsky.feed.getQuotes'
    AppBskyFeedGetRepostedBy: str = 'app.bsky.feed.getRepostedBy'
    AppBskyFeedGetSuggestedFeeds: str = 'app.bsky.feed.getSuggestedFeeds'
    AppBskyFeedGetTimeline: str = 'app.bsky.feed.getTimeline'
    AppBskyFeedLike: str = 'app.bsky.feed.like'
    AppBskyFeedPost: str = 'app.bsky.feed.post'
    AppBskyFeedPostgate: str = 'app.bsky.feed.postgate'
    AppBskyFeedRepost: str = 'app.bsky.feed.repost'
    AppBskyFeedSearchPosts: str = 'app.bsky.feed.searchPosts'
    AppBskyFeedSendInteractions: str = 'app.bsky.feed.sendInteractions'
    AppBskyFeedThreadgate: str = 'app.bsky.feed.threadgate'
    AppBskyGraphBlock: str = 'app.bsky.graph.block'
    AppBskyGraphDefs: str = 'app.bsky.graph.defs'
    AppBskyGraphFollow: str = 'app.bsky.graph.follow'
    AppBskyGraphGetActorStarterPacks: str = 'app.bsky.graph.getActorStarterPacks'
    AppBskyGraphGetBlocks: str = 'app.bsky.graph.getBlocks'
    AppBskyGraphGetFollowers: str = 'app.bsky.graph.getFollowers'
    AppBskyGraphGetFollows: str = 'app.bsky.graph.getFollows'
    AppBskyGraphGetKnownFollowers: str = 'app.bsky.graph.getKnownFollowers'
    AppBskyGraphGetList: str = 'app.bsky.graph.getList'
    AppBskyGraphGetListBlocks: str = 'app.bsky.graph.getListBlocks'
    AppBskyGraphGetListMutes: str = 'app.bsky.graph.getListMutes'
    AppBskyGraphGetLists: str = 'app.bsky.graph.getLists'
    AppBskyGraphGetMutes: str = 'app.bsky.graph.getMutes'
    AppBskyGraphGetRelationships: str = 'app.bsky.graph.getRelationships'
    AppBskyGraphGetStarterPack: str = 'app.bsky.graph.getStarterPack'
    AppBskyGraphGetStarterPacks: str = 'app.bsky.graph.getStarterPacks'
    AppBskyGraphGetSuggestedFollowsByActor: str = 'app.bsky.graph.getSuggestedFollowsByActor'
    AppBskyGraphList: str = 'app.bsky.graph.list'
    AppBskyGraphListblock: str = 'app.bsky.graph.listblock'
    AppBskyGraphListitem: str = 'app.bsky.graph.listitem'
    AppBskyGraphMuteActor: str = 'app.bsky.graph.muteActor'
    AppBskyGraphMuteActorList: str = 'app.bsky.graph.muteActorList'
    AppBskyGraphMuteThread: str = 'app.bsky.graph.muteThread'
    AppBskyGraphSearchStarterPacks: str = 'app.bsky.graph.searchStarterPacks'
    AppBskyGraphStarterpack: str = 'app.bsky.graph.starterpack'
    AppBskyGraphUnmuteActor: str = 'app.bsky.graph.unmuteActor'
    AppBskyGraphUnmuteActorList: str = 'app.bsky.graph.unmuteActorList'
    AppBskyGraphUnmuteThread: str = 'app.bsky.graph.unmuteThread'
    AppBskyGraphVerification: str = 'app.bsky.graph.verification'
    AppBskyLabelerDefs: str = 'app.bsky.labeler.defs'
    AppBskyLabelerGetServices: str = 'app.bsky.labeler.getServices'
    AppBskyLabelerService: str = 'app.bsky.labeler.service'
    AppBskyNotificationDefs: str = 'app.bsky.notification.defs'
    AppBskyNotificationGetUnreadCount: str = 'app.bsky.notification.getUnreadCount'
    AppBskyNotificationListNotifications: str = 'app.bsky.notification.listNotifications'
    AppBskyNotificationPutPreferences: str = 'app.bsky.notification.putPreferences'
    AppBskyNotificationRegisterPush: str = 'app.bsky.notification.registerPush'
    AppBskyNotificationUpdateSeen: str = 'app.bsky.notification.updateSeen'
    AppBskyRichtextFacet: str = 'app.bsky.richtext.facet'
    AppBskyUnspeccedDefs: str = 'app.bsky.unspecced.defs'
    AppBskyUnspeccedGetConfig: str = 'app.bsky.unspecced.getConfig'
    AppBskyUnspeccedGetPopularFeedGenerators: str = 'app.bsky.unspecced.getPopularFeedGenerators'
    AppBskyUnspeccedGetSuggestedFeeds: str = 'app.bsky.unspecced.getSuggestedFeeds'
    AppBskyUnspeccedGetSuggestedFeedsSkeleton: str = 'app.bsky.unspecced.getSuggestedFeedsSkeleton'
    AppBskyUnspeccedGetSuggestedStarterPacks: str = 'app.bsky.unspecced.getSuggestedStarterPacks'
    AppBskyUnspeccedGetSuggestedStarterPacksSkeleton: str = 'app.bsky.unspecced.getSuggestedStarterPacksSkeleton'
    AppBskyUnspeccedGetSuggestedUsers: str = 'app.bsky.unspecced.getSuggestedUsers'
    AppBskyUnspeccedGetSuggestedUsersSkeleton: str = 'app.bsky.unspecced.getSuggestedUsersSkeleton'
    AppBskyUnspeccedGetSuggestionsSkeleton: str = 'app.bsky.unspecced.getSuggestionsSkeleton'
    AppBskyUnspeccedGetTaggedSuggestions: str = 'app.bsky.unspecced.getTaggedSuggestions'
    AppBskyUnspeccedGetTrendingTopics: str = 'app.bsky.unspecced.getTrendingTopics'
    AppBskyUnspeccedGetTrends: str = 'app.bsky.unspecced.getTrends'
    AppBskyUnspeccedGetTrendsSkeleton: str = 'app.bsky.unspecced.getTrendsSkeleton'
    AppBskyUnspeccedSearchActorsSkeleton: str = 'app.bsky.unspecced.searchActorsSkeleton'
    AppBskyUnspeccedSearchPostsSkeleton: str = 'app.bsky.unspecced.searchPostsSkeleton'
    AppBskyUnspeccedSearchStarterPacksSkeleton: str = 'app.bsky.unspecced.searchStarterPacksSkeleton'
    AppBskyVideoDefs: str = 'app.bsky.video.defs'
    AppBskyVideoGetJobStatus: str = 'app.bsky.video.getJobStatus'
    AppBskyVideoGetUploadLimits: str = 'app.bsky.video.getUploadLimits'
    AppBskyVideoUploadVideo: str = 'app.bsky.video.uploadVideo'
    ChatBskyActorDeclaration: str = 'chat.bsky.actor.declaration'
    ChatBskyActorDefs: str = 'chat.bsky.actor.defs'
    ChatBskyActorDeleteAccount: str = 'chat.bsky.actor.deleteAccount'
    ChatBskyActorExportAccountData: str = 'chat.bsky.actor.exportAccountData'
    ChatBskyConvoAcceptConvo: str = 'chat.bsky.convo.acceptConvo'
    ChatBskyConvoAddReaction: str = 'chat.bsky.convo.addReaction'
    ChatBskyConvoDefs: str = 'chat.bsky.convo.defs'
    ChatBskyConvoDeleteMessageForSelf: str = 'chat.bsky.convo.deleteMessageForSelf'
    ChatBskyConvoGetConvo: str = 'chat.bsky.convo.getConvo'
    ChatBskyConvoGetConvoAvailability: str = 'chat.bsky.convo.getConvoAvailability'
    ChatBskyConvoGetConvoForMembers: str = 'chat.bsky.convo.getConvoForMembers'
    ChatBskyConvoGetLog: str = 'chat.bsky.convo.getLog'
    ChatBskyConvoGetMessages: str = 'chat.bsky.convo.getMessages'
    ChatBskyConvoLeaveConvo: str = 'chat.bsky.convo.leaveConvo'
    ChatBskyConvoListConvos: str = 'chat.bsky.convo.listConvos'
    ChatBskyConvoMuteConvo: str = 'chat.bsky.convo.muteConvo'
    ChatBskyConvoRemoveReaction: str = 'chat.bsky.convo.removeReaction'
    ChatBskyConvoSendMessage: str = 'chat.bsky.convo.sendMessage'
    ChatBskyConvoSendMessageBatch: str = 'chat.bsky.convo.sendMessageBatch'
    ChatBskyConvoUnmuteConvo: str = 'chat.bsky.convo.unmuteConvo'
    ChatBskyConvoUpdateAllRead: str = 'chat.bsky.convo.updateAllRead'
    ChatBskyConvoUpdateRead: str = 'chat.bsky.convo.updateRead'
    ChatBskyModerationGetActorMetadata: str = 'chat.bsky.moderation.getActorMetadata'
    ChatBskyModerationGetMessageContext: str = 'chat.bsky.moderation.getMessageContext'
    ChatBskyModerationUpdateActorAccess: str = 'chat.bsky.moderation.updateActorAccess'
    ComAtprotoAdminDefs: str = 'com.atproto.admin.defs'
    ComAtprotoAdminDeleteAccount: str = 'com.atproto.admin.deleteAccount'
    ComAtprotoAdminDisableAccountInvites: str = 'com.atproto.admin.disableAccountInvites'
    ComAtprotoAdminDisableInviteCodes: str = 'com.atproto.admin.disableInviteCodes'
    ComAtprotoAdminEnableAccountInvites: str = 'com.atproto.admin.enableAccountInvites'
    ComAtprotoAdminGetAccountInfo: str = 'com.atproto.admin.getAccountInfo'
    ComAtprotoAdminGetAccountInfos: str = 'com.atproto.admin.getAccountInfos'
    ComAtprotoAdminGetInviteCodes: str = 'com.atproto.admin.getInviteCodes'
    ComAtprotoAdminGetSubjectStatus: str = 'com.atproto.admin.getSubjectStatus'
    ComAtprotoAdminSearchAccounts: str = 'com.atproto.admin.searchAccounts'
    ComAtprotoAdminSendEmail: str = 'com.atproto.admin.sendEmail'
    ComAtprotoAdminUpdateAccountEmail: str = 'com.atproto.admin.updateAccountEmail'
    ComAtprotoAdminUpdateAccountHandle: str = 'com.atproto.admin.updateAccountHandle'
    ComAtprotoAdminUpdateAccountPassword: str = 'com.atproto.admin.updateAccountPassword'
    ComAtprotoAdminUpdateAccountSigningKey: str = 'com.atproto.admin.updateAccountSigningKey'
    ComAtprotoAdminUpdateSubjectStatus: str = 'com.atproto.admin.updateSubjectStatus'
    ComAtprotoIdentityDefs: str = 'com.atproto.identity.defs'
    ComAtprotoIdentityGetRecommendedDidCredentials: str = 'com.atproto.identity.getRecommendedDidCredentials'
    ComAtprotoIdentityRefreshIdentity: str = 'com.atproto.identity.refreshIdentity'
    ComAtprotoIdentityRequestPlcOperationSignature: str = 'com.atproto.identity.requestPlcOperationSignature'
    ComAtprotoIdentityResolveDid: str = 'com.atproto.identity.resolveDid'
    ComAtprotoIdentityResolveHandle: str = 'com.atproto.identity.resolveHandle'
    ComAtprotoIdentityResolveIdentity: str = 'com.atproto.identity.resolveIdentity'
    ComAtprotoIdentitySignPlcOperation: str = 'com.atproto.identity.signPlcOperation'
    ComAtprotoIdentitySubmitPlcOperation: str = 'com.atproto.identity.submitPlcOperation'
    ComAtprotoIdentityUpdateHandle: str = 'com.atproto.identity.updateHandle'
    ComAtprotoLabelDefs: str = 'com.atproto.label.defs'
    ComAtprotoLabelQueryLabels: str = 'com.atproto.label.queryLabels'
    ComAtprotoLabelSubscribeLabels: str = 'com.atproto.label.subscribeLabels'
    ComAtprotoLexiconSchema: str = 'com.atproto.lexicon.schema'
    ComAtprotoModerationCreateReport: str = 'com.atproto.moderation.createReport'
    ComAtprotoModerationDefs: str = 'com.atproto.moderation.defs'
    ComAtprotoRepoApplyWrites: str = 'com.atproto.repo.applyWrites'
    ComAtprotoRepoCreateRecord: str = 'com.atproto.repo.createRecord'
    ComAtprotoRepoDefs: str = 'com.atproto.repo.defs'
    ComAtprotoRepoDeleteRecord: str = 'com.atproto.repo.deleteRecord'
    ComAtprotoRepoDescribeRepo: str = 'com.atproto.repo.describeRepo'
    ComAtprotoRepoGetRecord: str = 'com.atproto.repo.getRecord'
    ComAtprotoRepoImportRepo: str = 'com.atproto.repo.importRepo'
    ComAtprotoRepoListMissingBlobs: str = 'com.atproto.repo.listMissingBlobs'
    ComAtprotoRepoListRecords: str = 'com.atproto.repo.listRecords'
    ComAtprotoRepoPutRecord: str = 'com.atproto.repo.putRecord'
    ComAtprotoRepoStrongRef: str = 'com.atproto.repo.strongRef'
    ComAtprotoRepoUploadBlob: str = 'com.atproto.repo.uploadBlob'
    ComAtprotoServerActivateAccount: str = 'com.atproto.server.activateAccount'
    ComAtprotoServerCheckAccountStatus: str = 'com.atproto.server.checkAccountStatus'
    ComAtprotoServerConfirmEmail: str = 'com.atproto.server.confirmEmail'
    ComAtprotoServerCreateAccount: str = 'com.atproto.server.createAccount'
    ComAtprotoServerCreateAppPassword: str = 'com.atproto.server.createAppPassword'
    ComAtprotoServerCreateInviteCode: str = 'com.atproto.server.createInviteCode'
    ComAtprotoServerCreateInviteCodes: str = 'com.atproto.server.createInviteCodes'
    ComAtprotoServerCreateSession: str = 'com.atproto.server.createSession'
    ComAtprotoServerDeactivateAccount: str = 'com.atproto.server.deactivateAccount'
    ComAtprotoServerDefs: str = 'com.atproto.server.defs'
    ComAtprotoServerDeleteAccount: str = 'com.atproto.server.deleteAccount'
    ComAtprotoServerDeleteSession: str = 'com.atproto.server.deleteSession'
    ComAtprotoServerDescribeServer: str = 'com.atproto.server.describeServer'
    ComAtprotoServerGetAccountInviteCodes: str = 'com.atproto.server.getAccountInviteCodes'
    ComAtprotoServerGetServiceAuth: str = 'com.atproto.server.getServiceAuth'
    ComAtprotoServerGetSession: str = 'com.atproto.server.getSession'
    ComAtprotoServerListAppPasswords: str = 'com.atproto.server.listAppPasswords'
    ComAtprotoServerRefreshSession: str = 'com.atproto.server.refreshSession'
    ComAtprotoServerRequestAccountDelete: str = 'com.atproto.server.requestAccountDelete'
    ComAtprotoServerRequestEmailConfirmation: str = 'com.atproto.server.requestEmailConfirmation'
    ComAtprotoServerRequestEmailUpdate: str = 'com.atproto.server.requestEmailUpdate'
    ComAtprotoServerRequestPasswordReset: str = 'com.atproto.server.requestPasswordReset'
    ComAtprotoServerReserveSigningKey: str = 'com.atproto.server.reserveSigningKey'
    ComAtprotoServerResetPassword: str = 'com.atproto.server.resetPassword'
    ComAtprotoServerRevokeAppPassword: str = 'com.atproto.server.revokeAppPassword'
    ComAtprotoServerUpdateEmail: str = 'com.atproto.server.updateEmail'
    ComAtprotoSyncDefs: str = 'com.atproto.sync.defs'
    ComAtprotoSyncGetBlob: str = 'com.atproto.sync.getBlob'
    ComAtprotoSyncGetBlocks: str = 'com.atproto.sync.getBlocks'
    ComAtprotoSyncGetCheckout: str = 'com.atproto.sync.getCheckout'
    ComAtprotoSyncGetHead: str = 'com.atproto.sync.getHead'
    ComAtprotoSyncGetHostStatus: str = 'com.atproto.sync.getHostStatus'
    ComAtprotoSyncGetLatestCommit: str = 'com.atproto.sync.getLatestCommit'
    ComAtprotoSyncGetRecord: str = 'com.atproto.sync.getRecord'
    ComAtprotoSyncGetRepo: str = 'com.atproto.sync.getRepo'
    ComAtprotoSyncGetRepoStatus: str = 'com.atproto.sync.getRepoStatus'
    ComAtprotoSyncListBlobs: str = 'com.atproto.sync.listBlobs'
    ComAtprotoSyncListHosts: str = 'com.atproto.sync.listHosts'
    ComAtprotoSyncListRepos: str = 'com.atproto.sync.listRepos'
    ComAtprotoSyncListReposByCollection: str = 'com.atproto.sync.listReposByCollection'
    ComAtprotoSyncNotifyOfUpdate: str = 'com.atproto.sync.notifyOfUpdate'
    ComAtprotoSyncRequestCrawl: str = 'com.atproto.sync.requestCrawl'
    ComAtprotoSyncSubscribeRepos: str = 'com.atproto.sync.subscribeRepos'
    ComAtprotoTempAddReservedHandle: str = 'com.atproto.temp.addReservedHandle'
    ComAtprotoTempCheckSignupQueue: str = 'com.atproto.temp.checkSignupQueue'
    ComAtprotoTempFetchLabels: str = 'com.atproto.temp.fetchLabels'
    ComAtprotoTempRequestPhoneVerification: str = 'com.atproto.temp.requestPhoneVerification'
    ToolsOzoneCommunicationCreateTemplate: str = 'tools.ozone.communication.createTemplate'
    ToolsOzoneCommunicationDefs: str = 'tools.ozone.communication.defs'
    ToolsOzoneCommunicationDeleteTemplate: str = 'tools.ozone.communication.deleteTemplate'
    ToolsOzoneCommunicationListTemplates: str = 'tools.ozone.communication.listTemplates'
    ToolsOzoneCommunicationUpdateTemplate: str = 'tools.ozone.communication.updateTemplate'
    ToolsOzoneModerationDefs: str = 'tools.ozone.moderation.defs'
    ToolsOzoneModerationEmitEvent: str = 'tools.ozone.moderation.emitEvent'
    ToolsOzoneModerationGetEvent: str = 'tools.ozone.moderation.getEvent'
    ToolsOzoneModerationGetRecord: str = 'tools.ozone.moderation.getRecord'
    ToolsOzoneModerationGetRecords: str = 'tools.ozone.moderation.getRecords'
    ToolsOzoneModerationGetRepo: str = 'tools.ozone.moderation.getRepo'
    ToolsOzoneModerationGetReporterStats: str = 'tools.ozone.moderation.getReporterStats'
    ToolsOzoneModerationGetRepos: str = 'tools.ozone.moderation.getRepos'
    ToolsOzoneModerationGetSubjects: str = 'tools.ozone.moderation.getSubjects'
    ToolsOzoneModerationQueryEvents: str = 'tools.ozone.moderation.queryEvents'
    ToolsOzoneModerationQueryStatuses: str = 'tools.ozone.moderation.queryStatuses'
    ToolsOzoneModerationSearchRepos: str = 'tools.ozone.moderation.searchRepos'
    ToolsOzoneServerGetConfig: str = 'tools.ozone.server.getConfig'
    ToolsOzoneSetAddValues: str = 'tools.ozone.set.addValues'
    ToolsOzoneSetDefs: str = 'tools.ozone.set.defs'
    ToolsOzoneSetDeleteSet: str = 'tools.ozone.set.deleteSet'
    ToolsOzoneSetDeleteValues: str = 'tools.ozone.set.deleteValues'
    ToolsOzoneSetGetValues: str = 'tools.ozone.set.getValues'
    ToolsOzoneSetQuerySets: str = 'tools.ozone.set.querySets'
    ToolsOzoneSetUpsertSet: str = 'tools.ozone.set.upsertSet'
    ToolsOzoneSettingDefs: str = 'tools.ozone.setting.defs'
    ToolsOzoneSettingListOptions: str = 'tools.ozone.setting.listOptions'
    ToolsOzoneSettingRemoveOptions: str = 'tools.ozone.setting.removeOptions'
    ToolsOzoneSettingUpsertOption: str = 'tools.ozone.setting.upsertOption'
    ToolsOzoneSignatureDefs: str = 'tools.ozone.signature.defs'
    ToolsOzoneSignatureFindCorrelation: str = 'tools.ozone.signature.findCorrelation'
    ToolsOzoneSignatureFindRelatedAccounts: str = 'tools.ozone.signature.findRelatedAccounts'
    ToolsOzoneSignatureSearchAccounts: str = 'tools.ozone.signature.searchAccounts'
    ToolsOzoneTeamAddMember: str = 'tools.ozone.team.addMember'
    ToolsOzoneTeamDefs: str = 'tools.ozone.team.defs'
    ToolsOzoneTeamDeleteMember: str = 'tools.ozone.team.deleteMember'
    ToolsOzoneTeamListMembers: str = 'tools.ozone.team.listMembers'
    ToolsOzoneTeamUpdateMember: str = 'tools.ozone.team.updateMember'


ids = _Ids()
load_models()



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`chat.bsky.moderation.getMessageContext`."""

    message_id: str  #: Message id.
    after: t.Optional[int] = 5  #: After.
    before: t.Optional[int] = 5  #: Before.
    convo_id: t.Optional[str] = (
        None  #: Conversation that the message is from. NOTE: this field will eventually be required.
    )


class ParamsDict(t.TypedDict):
    message_id: str  #: Message id.
    after: te.NotRequired[t.Optional[int]]  #: After.
    before: te.NotRequired[t.Optional[int]]  #: Before.
    convo_id: te.NotRequired[
        t.Optional[str]
    ]  #: Conversation that the message is from. NOTE: this field will eventually be required.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`chat.bsky.moderation.getMessageContext`."""

    messages: t.List[
        te.Annotated[
            t.Union['models.ChatBskyConvoDefs.MessageView', 'models.ChatBskyConvoDefs.DeletedMessageView'],
            Field(discriminator='py_type'),
        ]
    ]  #: Messages.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`com.atproto.sync.listHosts`."""

    cursor: t.Optional[str] = None  #: Cursor.
    limit: t.Optional[int] = Field(default=200, ge=1, le=1000)  #: Limit.


class ParamsDict(t.TypedDict):
    cursor: te.NotRequired[t.Optional[str]]  #: Cursor.
    limit: te.NotRequired[t.Optional[int]]  #: Limit.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.sync.listHosts`."""

    hosts: t.List[
        'models.ComAtprotoSyncListHosts.Host'
    ]  #: Sort order is not formally specified. Recommended order is by time host was first seen by the server, with oldest first.
    cursor: t.Optional[str] = None  #: Cursor.


class Host(base.ModelBase):
    """Definition model for :obj:`com.atproto.sync.listHosts`."""

    hostname: str  #: Hostname of server; not a URL (no scheme).
    account_count: t.Optional[int] = None  #: Account count.
    seq: t.Optional[int] = (
        None  #: Recent repo stream event sequence number. May be delayed from actual stream processing (eg, persisted cursor not in-memory cursor).
    )
    status: t.Optional['models.ComAtprotoSyncDefs.HostStatus'] = None  #: Status.

    py_type: t.Literal['com.atproto.sync.listHosts#host'] = Field(
        default='com.atproto.sync.listHosts#host', alias='$type', frozen=True
    )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
    from atproto_client.models.blob_ref import BlobRef
from atproto_client.models import base


class Record(base.RecordModelBase):
    """Record model for :obj:`app.bsky.feed.generator`."""

    created_at: string_formats.DateTime  #: Created at.
    did: string_formats.Did  #: Did.
    display_name: str = Field(max_length=240)  #: Display name.
    accepts_interactions: t.Optional[bool] = (
        None  #: Declaration that a feed accepts feedback interactions from a client through app.bsky.feed.sendInteractions.
    )
    avatar: t.Optional['BlobRef'] = None  #: Avatar.
    content_mode: t.Optional[
        t.Union['models.AppBskyFeedDefs.ContentModeUnspecified', 'models.AppBskyFeedDefs.ContentModeVideo', str]
    ] = None  #: Content mode.
    description: t.Optional[str] = Field(default=None, max_length=3000)  #: Description.
    description_facets: t.Optional[t.List['models.AppBskyRichtextFacet.Main']] = None  #: Description facets.
    labels: t.Optional[
        te.Annotated[t.Union['models.ComAtprotoLabelDefs.SelfLabels'], Field(default=None, discriminator='py_type')]
    ] = None  #: Self-label values.

    py_type: t.Literal['app.bsky.feed.generator'] = Field(default='app.bsky.feed.generator', alias='$type', frozen=True)


class GetRecordResponse(base.SugarResponseModelBase):
    """Get record response for :obj:`models.AppBskyFeedGenerator.Record`."""

    uri: str  #: The URI of the record.
    value: 'models.AppBskyFeedGenerator.Record'  #: The record.
    cid: t.Optional[str] = None  #: The CID of the record.


class ListRecordsResponse(base.SugarResponseModelBase):
    """List records response for :obj:`models.AppBskyFeedGenerator.Record`."""

    records: t.Dict[str, 'models.AppBskyFeedGenerator.Record']  #: Map of URIs to records.
    cursor: t.Optional[str] = None  #: Next page cursor.


class CreateRecordResponse(base.SugarResponseModelBase):
    """Create record response for :obj:`models.AppBskyFeedGenerator.Record`."""

    uri: str  #: The URI of the record.
    cid: str  #: The CID of the record.



import typing as t
from pathlib import Path

import click
from atproto_codegen.clients.generate_async_client import gen_client
from atproto_codegen.models.generator import generate_models
from atproto_codegen.namespaces.generator import generate_namespaces


class AliasedGroup(click.Group):
    """Ref: https://click.palletsprojects.com/en/8.1.x/advanced/."""

    def get_command(self, ctx: click.Context, cmd_name: str) -> t.Optional[click.Command]:
        rv = click.Group.get_command(self, ctx, cmd_name)
        if rv is not None:
            return rv

        matches = [x for x in self.list_commands(ctx) if x.startswith(cmd_name)]
        if not matches:
            return None
        if len(matches) == 1:
            return click.Group.get_command(self, ctx, matches[0])

        ctx.fail(f'Too many matches: {", ".join(sorted(matches))}')
        return None

    def resolve_command(
        self, ctx: click.Context, args: t.List[str]
    ) -> t.Tuple[t.Optional[str], t.Optional[click.Command], t.List[str]]:
        # always return the full command name
        _, cmd, args = super().resolve_command(ctx, args)

        name = None
        if cmd:
            name = cmd.name

        return name, cmd, args


def echo(ctx: click.Context, *args: t.Any) -> None:
    if not ctx.obj.get('silent'):
        click.echo(*args)


@click.group(cls=AliasedGroup)
@click.option('--silent', '-s', is_flag=True, default=False, help='Disable output.')
@click.pass_context
def atproto_cli(ctx: click.Context, silent: bool) -> None:
    """CLI of AT Protocol SDK for Python."""
    ctx.ensure_object(dict)
    ctx.obj['silent'] = silent


@atproto_cli.group(cls=AliasedGroup)
@click.option('--lexicon-dir', type=click.Path(exists=True), default=None, help='Path to dir with .JSON lexicon files.')
@click.pass_context
def gen(ctx: click.Context, lexicon_dir: t.Optional[str]) -> None:
    lexicon_dir_path = Path(lexicon_dir) if lexicon_dir else None
    ctx.obj['lexicon_dir'] = lexicon_dir_path


@gen.command(name='all', help='Generated models, namespaces, and async clients with default configs.')
@click.pass_context
def gen_all(ctx: click.Context) -> None:
    echo(ctx, 'Generating all:')

    echo(ctx, '- models...')
    _gen_models()
    echo(ctx, '- namespaces...')
    _gen_namespaces()
    echo(ctx, '- async clients...')
    _gen_async_version()

    echo(ctx, 'Done!')


def _gen_models(*args: t.Any) -> None:
    generate_models(*args)


def _gen_namespaces(*args: t.Any) -> None:
    generate_namespaces(*args)


def _gen_async_version() -> None:
    gen_client('client.py', 'async_client.py')


@gen.command(name='models')
@click.option('--output-dir', type=click.Path(exists=True), default=None)
@click.pass_context
def gen_models(ctx: click.Context, output_dir: t.Optional[str]) -> None:
    echo(ctx, 'Generating models...')

    if output_dir:
        # FIXME(MarshalX): remove hardcoded imports
        click.secho(
            "It doesn't work with '--output-dir' option very well because of hardcoded imports! Replace by yourself",
            fg='red',
        )
        _gen_models(ctx.obj.get('lexicon_dir'), Path(output_dir))
    else:
        _gen_models(ctx.obj.get('lexicon_dir'))

    echo(ctx, 'Done!')


@gen.command(name='namespaces')
@click.option('--output-dir', type=click.Path(exists=True), default=None)
@click.option('--async-filename', type=click.STRING, default=None, help='Should end with ".py".')
@click.option('--sync-filename', type=click.STRING, default=None, help='Should end with ".py".')
@click.pass_context
def gen_namespaces(
    ctx: click.Context, output_dir: t.Optional[str], async_filename: t.Optional[str], sync_filename: t.Optional[str]
) -> None:
    echo(ctx, 'Generating namespaces...')

    output_dir_path = Path(output_dir) if output_dir else None
    _gen_namespaces(ctx.obj.get('lexicon_dir'), output_dir_path, async_filename, sync_filename)

    echo(ctx, 'Done!')


@gen.command(name='async')
@click.pass_context
def gen_async_version(ctx: click.Context) -> None:
    echo(ctx, 'Generating async clients...')
    _gen_async_version()
    echo(ctx, 'Done!')


if __name__ == '__main__':
    atproto_cli()



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client.models.unknown_type import UnknownInputType, UnknownType
from atproto_client.models import base


class Data(base.DataModelBase):
    """Input data model for :obj:`com.atproto.server.createAccount`."""

    handle: string_formats.Handle  #: Requested handle for the account.
    did: t.Optional[string_formats.Did] = None  #: Pre-existing atproto DID, being imported to a new account.
    email: t.Optional[str] = None  #: Email.
    invite_code: t.Optional[str] = None  #: Invite code.
    password: t.Optional[str] = (
        None  #: Initial account password. May need to meet instance-specific password strength requirements.
    )
    plc_op: t.Optional['UnknownInputType'] = (
        None  #: A signed DID PLC operation to be submitted as part of importing an existing account to this instance. NOTE: this optional field may be updated when full account migration is implemented.
    )
    recovery_key: t.Optional[str] = (
        None  #: DID PLC rotation key (aka, recovery key) to be included in PLC creation operation.
    )
    verification_code: t.Optional[str] = None  #: Verification code.
    verification_phone: t.Optional[str] = None  #: Verification phone.


class DataDict(t.TypedDict):
    handle: string_formats.Handle  #: Requested handle for the account.
    did: te.NotRequired[t.Optional[string_formats.Did]]  #: Pre-existing atproto DID, being imported to a new account.
    email: te.NotRequired[t.Optional[str]]  #: Email.
    invite_code: te.NotRequired[t.Optional[str]]  #: Invite code.
    password: te.NotRequired[
        t.Optional[str]
    ]  #: Initial account password. May need to meet instance-specific password strength requirements.
    plc_op: te.NotRequired[
        t.Optional['UnknownInputType']
    ]  #: A signed DID PLC operation to be submitted as part of importing an existing account to this instance. NOTE: this optional field may be updated when full account migration is implemented.
    recovery_key: te.NotRequired[
        t.Optional[str]
    ]  #: DID PLC rotation key (aka, recovery key) to be included in PLC creation operation.
    verification_code: te.NotRequired[t.Optional[str]]  #: Verification code.
    verification_phone: te.NotRequired[t.Optional[str]]  #: Verification phone.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.server.createAccount`. Account login session returned on successful account creation."""

    access_jwt: str  #: Access jwt.
    did: string_formats.Did  #: The DID of the new account.
    handle: string_formats.Handle  #: Handle.
    refresh_jwt: str  #: Refresh jwt.
    did_doc: t.Optional['UnknownType'] = None  #: Complete DID document.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Data(base.DataModelBase):
    """Input data model for :obj:`tools.ozone.moderation.emitEvent`."""

    created_by: string_formats.Did  #: Created by.
    event: te.Annotated[
        t.Union[
            'models.ToolsOzoneModerationDefs.ModEventTakedown',
            'models.ToolsOzoneModerationDefs.ModEventAcknowledge',
            'models.ToolsOzoneModerationDefs.ModEventEscalate',
            'models.ToolsOzoneModerationDefs.ModEventComment',
            'models.ToolsOzoneModerationDefs.ModEventLabel',
            'models.ToolsOzoneModerationDefs.ModEventReport',
            'models.ToolsOzoneModerationDefs.ModEventMute',
            'models.ToolsOzoneModerationDefs.ModEventUnmute',
            'models.ToolsOzoneModerationDefs.ModEventMuteReporter',
            'models.ToolsOzoneModerationDefs.ModEventUnmuteReporter',
            'models.ToolsOzoneModerationDefs.ModEventReverseTakedown',
            'models.ToolsOzoneModerationDefs.ModEventResolveAppeal',
            'models.ToolsOzoneModerationDefs.ModEventEmail',
            'models.ToolsOzoneModerationDefs.ModEventDivert',
            'models.ToolsOzoneModerationDefs.ModEventTag',
            'models.ToolsOzoneModerationDefs.AccountEvent',
            'models.ToolsOzoneModerationDefs.IdentityEvent',
            'models.ToolsOzoneModerationDefs.RecordEvent',
            'models.ToolsOzoneModerationDefs.ModEventPriorityScore',
        ],
        Field(discriminator='py_type'),
    ]  #: Event.
    subject: te.Annotated[
        t.Union['models.ComAtprotoAdminDefs.RepoRef', 'models.ComAtprotoRepoStrongRef.Main'],
        Field(discriminator='py_type'),
    ]  #: Subject.
    subject_blob_cids: t.Optional[t.List[string_formats.Cid]] = None  #: Subject blob cids.


class DataDict(t.TypedDict):
    created_by: string_formats.Did  #: Created by.
    event: te.Annotated[
        t.Union[
            'models.ToolsOzoneModerationDefs.ModEventTakedown',
            'models.ToolsOzoneModerationDefs.ModEventAcknowledge',
            'models.ToolsOzoneModerationDefs.ModEventEscalate',
            'models.ToolsOzoneModerationDefs.ModEventComment',
            'models.ToolsOzoneModerationDefs.ModEventLabel',
            'models.ToolsOzoneModerationDefs.ModEventReport',
            'models.ToolsOzoneModerationDefs.ModEventMute',
            'models.ToolsOzoneModerationDefs.ModEventUnmute',
            'models.ToolsOzoneModerationDefs.ModEventMuteReporter',
            'models.ToolsOzoneModerationDefs.ModEventUnmuteReporter',
            'models.ToolsOzoneModerationDefs.ModEventReverseTakedown',
            'models.ToolsOzoneModerationDefs.ModEventResolveAppeal',
            'models.ToolsOzoneModerationDefs.ModEventEmail',
            'models.ToolsOzoneModerationDefs.ModEventDivert',
            'models.ToolsOzoneModerationDefs.ModEventTag',
            'models.ToolsOzoneModerationDefs.AccountEvent',
            'models.ToolsOzoneModerationDefs.IdentityEvent',
            'models.ToolsOzoneModerationDefs.RecordEvent',
            'models.ToolsOzoneModerationDefs.ModEventPriorityScore',
        ],
        Field(discriminator='py_type'),
    ]  #: Event.
    subject: te.Annotated[
        t.Union['models.ComAtprotoAdminDefs.RepoRef', 'models.ComAtprotoRepoStrongRef.Main'],
        Field(discriminator='py_type'),
    ]  #: Subject.
    subject_blob_cids: te.NotRequired[t.Optional[t.List[string_formats.Cid]]]  #: Subject blob cids.



import typing as t
from dataclasses import dataclass

import libipld
import typing_extensions as te
from pydantic import GetJsonSchemaHandler
from pydantic.json_schema import JsonSchemaValue
from pydantic_core import core_schema


@dataclass
class Multihash:
    code: int
    size: int
    digest: bytes


@dataclass(eq=False)
class CID:
    version: int
    codec: int
    hash: Multihash

    _stringified_form: t.Optional[str] = None
    _raw_byte_form: t.Optional[str] = None

    @classmethod
    def decode(cls, value: t.Union[str, bytes]) -> 'CID':
        cid = libipld.decode_cid(value)

        multihash = Multihash(
            code=cid['hash']['code'],
            size=cid['hash']['size'],
            digest=cid['hash']['digest'],
        )

        instance = cls(
            version=cid['version'],
            codec=cid['codec'],
            hash=multihash,
        )

        if isinstance(value, str):
            instance._stringified_form = value
        else:
            instance._raw_byte_form = value

        return instance

    def encode(self) -> str:
        if self._stringified_form is not None:
            return self._stringified_form

        self._stringified_form = libipld.encode_cid(self._raw_byte_form)
        return self._stringified_form

    def __str__(self) -> str:
        return self.encode()

    def __hash__(self) -> int:
        return hash(self.encode())

    def __eq__(self, other: t.Any) -> bool:
        if isinstance(other, str):
            return self.encode() == other

        if isinstance(other, CID):
            return self.encode() == other.encode()

        return False


class _CIDPydanticAnnotation:
    @classmethod
    def __get_pydantic_core_schema__(
        cls,
        _source_type: t.Any,
        _handler: t.Callable[[t.Any], core_schema.CoreSchema],
    ) -> core_schema.CoreSchema:
        """We return a pydantic_core.CoreSchema that behaves in the following ways below.

        * Strings and bytes will be parsed as `CID` instances
        * `CID` instances will be parsed as `CID` instances without any changes
        * Nothing else will pass validation
        * Serialization will always return just a str
        """

        def validate_from_value(value: t.Union[str, bytes]) -> CID:
            return CID.decode(value)

        from_str_schema = core_schema.chain_schema(
            [
                core_schema.str_schema(),
                core_schema.no_info_plain_validator_function(validate_from_value),
            ]
        )

        from_bytes_schema = core_schema.chain_schema(
            [
                core_schema.bytes_schema(),
                core_schema.no_info_plain_validator_function(validate_from_value),
            ]
        )

        return core_schema.json_or_python_schema(
            json_schema=from_str_schema,
            python_schema=core_schema.union_schema(
                [
                    # check if it's an instance first before doing any further work
                    core_schema.is_instance_schema(CID),
                    from_str_schema,
                    from_bytes_schema,
                ]
            ),
            serialization=core_schema.plain_serializer_function_ser_schema(lambda instance: instance.encode()),
        )

    @classmethod
    def __get_pydantic_json_schema__(
        cls, _core_schema: core_schema.CoreSchema, handler: GetJsonSchemaHandler
    ) -> JsonSchemaValue:
        # Use the same schema that would be used for `str`
        return handler(core_schema.str_schema())


CIDType = te.Annotated[CID, _CIDPydanticAnnotation]



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
    from atproto_client.models.blob_ref import BlobRef
from atproto_client.models import base


class Main(base.ModelBase):
    """Definition model for :obj:`app.bsky.embed.video`."""

    video: 'BlobRef'  #: The mp4 video file. May be up to 100mb, formerly limited to 50mb.
    alt: t.Optional[str] = Field(
        default=None, max_length=10000
    )  #: Alt text description of the video, for accessibility.
    aspect_ratio: t.Optional['models.AppBskyEmbedDefs.AspectRatio'] = None  #: Aspect ratio.
    captions: t.Optional[t.List['models.AppBskyEmbedVideo.Caption']] = Field(default=None, max_length=20)  #: Captions.

    py_type: t.Literal['app.bsky.embed.video'] = Field(default='app.bsky.embed.video', alias='$type', frozen=True)


class Caption(base.ModelBase):
    """Definition model for :obj:`app.bsky.embed.video`."""

    file: 'BlobRef'  #: File.
    lang: string_formats.Language  #: Lang.

    py_type: t.Literal['app.bsky.embed.video#caption'] = Field(
        default='app.bsky.embed.video#caption', alias='$type', frozen=True
    )


class View(base.ModelBase):
    """Definition model for :obj:`app.bsky.embed.video`."""

    cid: string_formats.Cid  #: Cid.
    playlist: string_formats.Uri  #: Playlist.
    alt: t.Optional[str] = Field(default=None, max_length=10000)  #: Alt.
    aspect_ratio: t.Optional['models.AppBskyEmbedDefs.AspectRatio'] = None  #: Aspect ratio.
    thumbnail: t.Optional[string_formats.Uri] = None  #: Thumbnail.

    py_type: t.Literal['app.bsky.embed.video#view'] = Field(
        default='app.bsky.embed.video#view', alias='$type', frozen=True
    )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te

from atproto_client.models import base, string_formats


class Data(base.DataModelBase):
    """Input data model for :obj:`tools.ozone.communication.updateTemplate`."""

    id: str  #: ID of the template to be updated.
    content_markdown: t.Optional[str] = (
        None  #: Content of the template, markdown supported, can contain variable placeholders.
    )
    disabled: t.Optional[bool] = None  #: Disabled.
    lang: t.Optional[string_formats.Language] = None  #: Message language.
    name: t.Optional[str] = None  #: Name of the template.
    subject: t.Optional[str] = None  #: Subject of the message, used in emails.
    updated_by: t.Optional[string_formats.Did] = None  #: DID of the user who is updating the template.


class DataDict(t.TypedDict):
    id: str  #: ID of the template to be updated.
    content_markdown: te.NotRequired[
        t.Optional[str]
    ]  #: Content of the template, markdown supported, can contain variable placeholders.
    disabled: te.NotRequired[t.Optional[bool]]  #: Disabled.
    lang: te.NotRequired[t.Optional[string_formats.Language]]  #: Message language.
    name: te.NotRequired[t.Optional[str]]  #: Name of the template.
    subject: te.NotRequired[t.Optional[str]]  #: Subject of the message, used in emails.
    updated_by: te.NotRequired[t.Optional[string_formats.Did]]  #: DID of the user who is updating the template.



import typing as t

from atproto_client import models
from atproto_client.models.utils import get_model_as_dict, get_or_create

from atproto_firehose.client import AsyncFirehoseClient, FirehoseClient

if t.TYPE_CHECKING:
    from atproto_firehose.models import MessageFrame

_REPOS_BASE_WEBSOCKET_URI = 'wss://bsky.network/xrpc'
_LABELS_BASE_WEBSOCKET_URI = 'wss://mod.bsky.app/xrpc'

_REPOS_DEFAULT_RECV_TIMEOUT = 30.0
_LABELS_DEFAULT_RECV_TIMEOUT = 300.0  # 5 minutes

# TODO(MarshalX): Everything here could be autogenerated from the lexicon.
_SUBSCRIBE_REPOS_MESSAGE_TYPE_TO_MODEL = {
    '#commit': models.ComAtprotoSyncSubscribeRepos.Commit,
    '#info': models.ComAtprotoSyncSubscribeRepos.Info,
    '#identity': models.ComAtprotoSyncSubscribeRepos.Identity,  # DEPRECATED
    '#account': models.ComAtprotoSyncSubscribeRepos.Account,
    '#sync': models.ComAtprotoSyncSubscribeRepos.Sync,
}
_SUBSCRIBE_LABELS_MESSAGE_TYPE_TO_MODEL = {
    '#labels': models.ComAtprotoLabelSubscribeLabels.Labels,
    '#info': models.ComAtprotoLabelSubscribeLabels.Info,
}

#: Subscribe Repos Message
SubscribeReposMessage = t.Union[
    models.ComAtprotoSyncSubscribeRepos.Commit,
    models.ComAtprotoSyncSubscribeRepos.Info,
    models.ComAtprotoSyncSubscribeRepos.Identity,
    models.ComAtprotoSyncSubscribeRepos.Account,
    models.ComAtprotoSyncSubscribeRepos.Sync,
]
#: Subscribe Labels Message
SubscribeLabelsMessage = t.Union[
    models.ComAtprotoLabelSubscribeLabels.Labels,
    models.ComAtprotoLabelSubscribeLabels.Info,
]


def parse_subscribe_repos_message(message: 'MessageFrame') -> SubscribeReposMessage:
    """Parse Firehose repositories message to the corresponding model.

    Note:
        Use `decode_inner_cbor` only when required to increase performance.

    Args:
        message: Message frame.

    Returns:
        :obj:`SubscribeReposMessage`: Corresponding message model.
    """
    model_class = _SUBSCRIBE_REPOS_MESSAGE_TYPE_TO_MODEL[message.type]
    return get_or_create(message.body, model_class)


def parse_subscribe_labels_message(message: 'MessageFrame') -> SubscribeLabelsMessage:
    """Parse Firehose labels message to the corresponding model.

    Args:
        message: Message frame.

    Returns:
        :obj:`SubscribeLabelsMessage`: Corresponding message model.
    """
    model_class = _SUBSCRIBE_LABELS_MESSAGE_TYPE_TO_MODEL[message.type]
    return get_or_create(message.body, model_class)


class FirehoseSubscribeReposClient(FirehoseClient):
    """Firehose subscribe repos client.

    Args:
        params: Parameters model.
        base_uri: Base websocket URI. Example: `wss://bsky.social/xrpc`.
        recv_timeout: Reconnect to the server after this many seconds of inactivity.
            Default is 30 seconds.
    """

    def __init__(
        self,
        params: t.Optional[t.Union[dict, 'models.ComAtprotoSyncSubscribeRepos.Params']] = None,
        base_uri: t.Optional[str] = _REPOS_BASE_WEBSOCKET_URI,
        recv_timeout: t.Optional[float] = _REPOS_DEFAULT_RECV_TIMEOUT,
    ) -> None:
        params_model = get_or_create(params, models.ComAtprotoSyncSubscribeRepos.Params)

        params_dict = None
        if params_model:
            params_dict = get_model_as_dict(params_model)

        super().__init__(
            method='com.atproto.sync.subscribeRepos', base_uri=base_uri, params=params_dict, recv_timeout=recv_timeout
        )


class AsyncFirehoseSubscribeReposClient(AsyncFirehoseClient):
    """Async firehose subscribe repos client.

    Args:
        params: Parameters model.
        base_uri: Base websocket URI. Example: `wss://bsky.social/xrpc`.
        recv_timeout: Reconnect to the server after this many seconds of inactivity.
            Default is 30 seconds.
    """

    def __init__(
        self,
        params: t.Optional[t.Union[dict, 'models.ComAtprotoSyncSubscribeRepos.Params']] = None,
        base_uri: t.Optional[str] = _REPOS_BASE_WEBSOCKET_URI,
        recv_timeout: t.Optional[float] = _REPOS_DEFAULT_RECV_TIMEOUT,
    ) -> None:
        params_model = get_or_create(params, models.ComAtprotoSyncSubscribeRepos.Params)

        params_dict = None
        if params_model:
            params_dict = get_model_as_dict(params_model)

        super().__init__(
            method='com.atproto.sync.subscribeRepos', base_uri=base_uri, params=params_dict, recv_timeout=recv_timeout
        )


class FirehoseSubscribeLabelsClient(FirehoseClient):
    """Firehose subscribe labels client.

    Args:
        params: Parameters model.
        base_uri: Base websocket URI. Example: `wss://bsky.social/xrpc`.
        recv_timeout: Reconnect to the server after this many seconds of inactivity.
            Default is 300 seconds (5 minutes).
    """

    def __init__(
        self,
        params: t.Optional[t.Union[dict, 'models.ComAtprotoLabelSubscribeLabels.Params']] = None,
        base_uri: t.Optional[str] = _LABELS_BASE_WEBSOCKET_URI,
        recv_timeout: t.Optional[float] = _LABELS_DEFAULT_RECV_TIMEOUT,
    ) -> None:
        params_model = get_or_create(params, models.ComAtprotoLabelSubscribeLabels.Params)

        params_dict = None
        if params_model:
            params_dict = get_model_as_dict(params_model)

        super().__init__(
            method='com.atproto.label.subscribeLabels', base_uri=base_uri, params=params_dict, recv_timeout=recv_timeout
        )


class AsyncFirehoseSubscribeLabelsClient(AsyncFirehoseClient):
    """Async firehose subscribe labels client.

    Args:
        params: Parameters model.
        base_uri: Base websocket URI. Example: `wss://bsky.social/xrpc`.
        recv_timeout: Reconnect to the server after this many seconds of inactivity.
            Default is 300 seconds (5 minutes).
    """

    def __init__(
        self,
        params: t.Optional[t.Union[dict, 'models.ComAtprotoLabelSubscribeLabels.Params']] = None,
        base_uri: t.Optional[str] = _LABELS_BASE_WEBSOCKET_URI,
        recv_timeout: t.Optional[float] = _LABELS_DEFAULT_RECV_TIMEOUT,
    ) -> None:
        params_model = get_or_create(params, models.ComAtprotoLabelSubscribeLabels.Params)

        params_dict = None
        if params_model:
            params_dict = get_model_as_dict(params_model)

        super().__init__(
            method='com.atproto.label.subscribeLabels', base_uri=base_uri, params=params_dict, recv_timeout=recv_timeout
        )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`app.bsky.unspecced.getSuggestionsSkeleton`."""

    cursor: t.Optional[str] = None  #: Cursor.
    limit: t.Optional[int] = Field(default=50, ge=1, le=100)  #: Limit.
    relative_to_did: t.Optional[string_formats.Did] = (
        None  #: DID of the account to get suggestions relative to. If not provided, suggestions will be based on the viewer.
    )
    viewer: t.Optional[string_formats.Did] = (
        None  #: DID of the account making the request (not included for public/unauthenticated queries). Used to boost followed accounts in ranking.
    )


class ParamsDict(t.TypedDict):
    cursor: te.NotRequired[t.Optional[str]]  #: Cursor.
    limit: te.NotRequired[t.Optional[int]]  #: Limit.
    relative_to_did: te.NotRequired[
        t.Optional[string_formats.Did]
    ]  #: DID of the account to get suggestions relative to. If not provided, suggestions will be based on the viewer.
    viewer: te.NotRequired[
        t.Optional[string_formats.Did]
    ]  #: DID of the account making the request (not included for public/unauthenticated queries). Used to boost followed accounts in ranking.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`app.bsky.unspecced.getSuggestionsSkeleton`."""

    actors: t.List['models.AppBskyUnspeccedDefs.SkeletonSearchActor']  #: Actors.
    cursor: t.Optional[str] = None  #: Cursor.
    rec_id: t.Optional[int] = None  #: Snowflake for this recommendation, use when submitting recommendation events.
    relative_to_did: t.Optional[string_formats.Did] = (
        None  #: DID of the account these suggestions are relative to. If this is returned undefined, suggestions are based on the viewer.
    )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Data(base.DataModelBase):
    """Input data model for :obj:`chat.bsky.convo.sendMessageBatch`."""

    items: t.List['models.ChatBskyConvoSendMessageBatch.BatchItem'] = Field(max_length=100)  #: Items.


class DataDict(t.TypedDict):
    items: t.List['models.ChatBskyConvoSendMessageBatch.BatchItem']  #: Items.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`chat.bsky.convo.sendMessageBatch`."""

    items: t.List['models.ChatBskyConvoDefs.MessageView']  #: Items.


class BatchItem(base.ModelBase):
    """Definition model for :obj:`chat.bsky.convo.sendMessageBatch`."""

    convo_id: str  #: Convo id.
    message: 'models.ChatBskyConvoDefs.MessageInput'  #: Message.

    py_type: t.Literal['chat.bsky.convo.sendMessageBatch#batchItem'] = Field(
        default='chat.bsky.convo.sendMessageBatch#batchItem', alias='$type', frozen=True
    )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`chat.bsky.moderation.getActorMetadata`."""

    actor: string_formats.Did  #: Actor.


class ParamsDict(t.TypedDict):
    actor: string_formats.Did  #: Actor.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`chat.bsky.moderation.getActorMetadata`."""

    all: 'models.ChatBskyModerationGetActorMetadata.Metadata'  #: All.
    day: 'models.ChatBskyModerationGetActorMetadata.Metadata'  #: Day.
    month: 'models.ChatBskyModerationGetActorMetadata.Metadata'  #: Month.


class Metadata(base.ModelBase):
    """Definition model for :obj:`chat.bsky.moderation.getActorMetadata`."""

    convos: int  #: Convos.
    convos_started: int  #: Convos started.
    messages_received: int  #: Messages received.
    messages_sent: int  #: Messages sent.

    py_type: t.Literal['chat.bsky.moderation.getActorMetadata#metadata'] = Field(
        default='chat.bsky.moderation.getActorMetadata#metadata', alias='$type', frozen=True
    )



import typing as t

if t.TYPE_CHECKING:
    from atproto_client.client.async_raw import AsyncClientRaw
    from atproto_client.client.raw import ClientRaw


class NamespaceBase:
    def __init__(self, client: 'ClientRaw') -> None:
        self._client: 'ClientRaw' = client


class AsyncNamespaceBase:
    def __init__(self, client: 'AsyncClientRaw') -> None:
        self._client: 'AsyncClientRaw' = client


class RecordBase:
    def __init__(self, client: 'ClientRaw') -> None:
        self._client: 'ClientRaw' = client


class AsyncRecordBase:
    def __init__(self, client: 'AsyncClientRaw') -> None:
        self._client: 'AsyncClientRaw' = client



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`app.bsky.feed.getFeedGenerator`."""

    feed: string_formats.AtUri  #: AT-URI of the feed generator record.


class ParamsDict(t.TypedDict):
    feed: string_formats.AtUri  #: AT-URI of the feed generator record.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`app.bsky.feed.getFeedGenerator`."""

    is_online: (
        bool  #: Indicates whether the feed generator service has been online recently, or else seems to be inactive.
    )
    is_valid: bool  #: Indicates whether the feed generator service is compatible with the record declaration.
    view: 'models.AppBskyFeedDefs.GeneratorView'  #: View.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from atproto_client.models import base, string_formats


class Data(base.DataModelBase):
    """Input data model for :obj:`app.bsky.notification.registerPush`."""

    app_id: str  #: App id.
    platform: t.Union[t.Literal['ios'], t.Literal['android'], t.Literal['web'], str]  #: Platform.
    service_did: string_formats.Did  #: Service did.
    token: str  #: Token.


class DataDict(t.TypedDict):
    app_id: str  #: App id.
    platform: t.Union[t.Literal['ios'], t.Literal['android'], t.Literal['web'], str]  #: Platform.
    service_did: string_formats.Did  #: Service did.
    token: str  #: Token.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
    from atproto_client.models.unknown_type import UnknownType
from atproto_client.models import base


class Option(base.ModelBase):
    """Definition model for :obj:`tools.ozone.setting.defs`."""

    created_by: string_formats.Did  #: Created by.
    did: string_formats.Did  #: Did.
    key: string_formats.Nsid  #: Key.
    last_updated_by: string_formats.Did  #: Last updated by.
    scope: t.Union[t.Literal['instance'], t.Literal['personal'], str]  #: Scope.
    value: 'UnknownType'  #: Value.
    created_at: t.Optional[string_formats.DateTime] = None  #: Created at.
    description: t.Optional[str] = Field(default=None, max_length=10240)  #: Description.
    manager_role: t.Optional[
        t.Union[
            'models.ToolsOzoneTeamDefs.RoleModerator',
            'models.ToolsOzoneTeamDefs.RoleTriage',
            'models.ToolsOzoneTeamDefs.RoleAdmin',
            str,
        ]
    ] = None  #: Manager role.
    updated_at: t.Optional[string_formats.DateTime] = None  #: Updated at.

    py_type: t.Literal['tools.ozone.setting.defs#option'] = Field(
        default='tools.ozone.setting.defs#option', alias='$type', frozen=True
    )



from atproto_client.exceptions import *
from atproto_core.exceptions import *
from atproto_crypto.exceptions import *
from atproto_firehose.exceptions import *
from atproto_identity.exceptions import *
from atproto_lexicon.exceptions import *
from atproto_server.exceptions import *



import re
import typing as t
import urllib.parse as urlparse
from urllib.parse import urlencode

from atproto_core.exceptions import InvalidAtUriError

# ref: https://github.com/bluesky-social/atproto/blob/a4af3494bd8263187bb3450193bcd2467ae6b788/packages/syntax/src/aturi.ts#L5C5-L5C98
_ATP_URI_REGEX = r'^(at:\/\/)?((?:did:[a-z0-9:%-]+)|(?:[a-z0-9][a-z0-9.:-]*))(\/[^?#\s]*)?(\?[^#\s]+)?(#[^\s]+)?$'
_ATP_RELATIVE_URI_REGEX = r'^(\/[^?#\s]*)?\?([^#\s]+)?(#[^\s]+)?$'  # modified to not include "?" in a group


class AtUri:
    """ATP URI Scheme.

    Examples:
        Repository: at://alice.host.com

        Repository: at://did:plc:bv6ggog3tya2z3vxsub7hnal

        Collection: at://alice.host.com/io.example.song

        Record: at://alice.host.com/io.example.song/3yI5-c1z-cc2p-1a

        Record Field: at://alice.host.com/io.example.song/3yI5-c1z-cc2p-1a#/title
    """

    def __init__(
        self,
        host: str,
        pathname: str = '',
        hash_: str = '',
        search_params: t.Optional[t.List[t.Tuple[str, t.Any]]] = None,
    ) -> None:
        if search_params is None:
            search_params = []

        if hash_ and not pathname:
            raise InvalidAtUriError('`hash_` cannot be set without `pathname`')
        if search_params and not pathname:
            raise InvalidAtUriError('`search_params` cannot be set without `pathname`')

        self.host = host
        self.pathname = pathname
        self.hash = hash_
        self.search_params = search_params

    @property
    def protocol(self) -> str:
        """Get protocol."""
        return 'at://'

    @property
    def origin(self) -> str:
        """Get origin."""
        return f'at://{self.host}'

    @property
    def hostname(self) -> str:
        """Get hostname."""
        return self.host

    @property
    def collection(self) -> str:
        """Get collection name."""
        for part in self.pathname.split('/'):
            if part:
                return part

        return ''

    @property
    def rkey(self) -> str:
        """Get record key (rkey)."""
        parts = [p for p in self.pathname.split('/') if p]
        if len(parts) > 1:
            return parts[1]

        return ''

    @property
    def http(self) -> str:
        """Convert instance to HTTP URI."""
        return str(self)

    @property
    def search(self) -> str:
        """Get search params."""
        return urlencode(self.search_params)

    @classmethod
    def make(cls, handle_or_did: str, collection: t.Optional[str] = None, rkey: t.Optional[str] = None) -> 'AtUri':
        """Create `AtUri` instance from handle or DID."""
        uri = handle_or_did

        if collection:
            uri = f'{handle_or_did}/{collection}'
        if rkey:
            uri = f'{uri}/{rkey}'

        return cls.from_str(uri)

    @classmethod
    def from_str(cls, uri: str) -> 'AtUri':
        """Create `AtUri` instance from URI."""
        groups = re.findall(_ATP_URI_REGEX, uri, re.IGNORECASE)
        if not groups:
            raise InvalidAtUriError

        group = groups[0]
        search_params = urlparse.parse_qsl(group[3] or '')
        return cls(host=group[1] or '', pathname=group[2] or '', hash_=group[4] or '', search_params=search_params)

    @classmethod
    def from_relative_str(cls, base: str, uri: str) -> 'AtUri':
        """Create `AtUri` instance from relative URI."""
        groups = re.findall(_ATP_RELATIVE_URI_REGEX, uri, re.IGNORECASE)
        if not groups:
            raise InvalidAtUriError

        group = groups[0]
        search_params = urlparse.parse_qsl(group[1] or '')
        return cls(host=base, pathname=group[0] or '', hash_=group[2] or '', search_params=search_params)

    def __str__(self) -> str:
        path = self.pathname or '/'
        if not path.startswith('/'):
            path = f'/{path}'

        query = urlencode(self.search_params)
        if query:
            query = f'?{query}'

        hash_ = self.hash
        if hash_ and not hash_.startswith('#'):
            hash_ = f'#{hash_}'

        return f'at://{self.host}{path}{query}{hash_}'

    def __hash__(self) -> int:
        return hash(str(self))

    def __eq__(self, other: t.Any) -> bool:
        if isinstance(other, AtUri):
            return hash(self) == hash(other)

        return False



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`com.atproto.sync.getHostStatus`."""

    hostname: str  #: Hostname of the host (eg, PDS or relay) being queried.


class ParamsDict(t.TypedDict):
    hostname: str  #: Hostname of the host (eg, PDS or relay) being queried.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.sync.getHostStatus`."""

    hostname: str  #: Hostname.
    account_count: t.Optional[int] = (
        None  #: Number of accounts on the server which are associated with the upstream host. Note that the upstream may actually have more accounts.
    )
    seq: t.Optional[int] = (
        None  #: Recent repo stream event sequence number. May be delayed from actual stream processing (eg, persisted cursor not in-memory cursor).
    )
    status: t.Optional['models.ComAtprotoSyncDefs.HostStatus'] = None  #: Status.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`app.bsky.unspecced.searchStarterPacksSkeleton`."""

    q: str  #: Search query string; syntax, phrase, boolean, and faceting is unspecified, but Lucene query syntax is recommended.
    cursor: t.Optional[str] = (
        None  #: Optional pagination mechanism; may not necessarily allow scrolling through entire result set.
    )
    limit: t.Optional[int] = Field(default=25, ge=1, le=100)  #: Limit.
    viewer: t.Optional[string_formats.Did] = (
        None  #: DID of the account making the request (not included for public/unauthenticated queries).
    )


class ParamsDict(t.TypedDict):
    q: str  #: Search query string; syntax, phrase, boolean, and faceting is unspecified, but Lucene query syntax is recommended.
    cursor: te.NotRequired[
        t.Optional[str]
    ]  #: Optional pagination mechanism; may not necessarily allow scrolling through entire result set.
    limit: te.NotRequired[t.Optional[int]]  #: Limit.
    viewer: te.NotRequired[
        t.Optional[string_formats.Did]
    ]  #: DID of the account making the request (not included for public/unauthenticated queries).


class Response(base.ResponseModelBase):
    """Output data model for :obj:`app.bsky.unspecced.searchStarterPacksSkeleton`."""

    starter_packs: t.List['models.AppBskyUnspeccedDefs.SkeletonSearchStarterPack']  #: Starter packs.
    cursor: t.Optional[str] = None  #: Cursor.
    hits_total: t.Optional[int] = (
        None  #: Count of search hits. Optional, may be rounded/truncated, and may not be possible to paginate through all hits.
    )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class LabelerView(base.ModelBase):
    """Definition model for :obj:`app.bsky.labeler.defs`."""

    cid: string_formats.Cid  #: Cid.
    creator: 'models.AppBskyActorDefs.ProfileView'  #: Creator.
    indexed_at: string_formats.DateTime  #: Indexed at.
    uri: string_formats.AtUri  #: Uri.
    labels: t.Optional[t.List['models.ComAtprotoLabelDefs.Label']] = None  #: Labels.
    like_count: t.Optional[int] = Field(default=None, ge=0)  #: Like count.
    viewer: t.Optional['models.AppBskyLabelerDefs.LabelerViewerState'] = None  #: Viewer.

    py_type: t.Literal['app.bsky.labeler.defs#labelerView'] = Field(
        default='app.bsky.labeler.defs#labelerView', alias='$type', frozen=True
    )


class LabelerViewDetailed(base.ModelBase):
    """Definition model for :obj:`app.bsky.labeler.defs`."""

    cid: string_formats.Cid  #: Cid.
    creator: 'models.AppBskyActorDefs.ProfileView'  #: Creator.
    indexed_at: string_formats.DateTime  #: Indexed at.
    policies: 'models.AppBskyLabelerDefs.LabelerPolicies'  #: Policies.
    uri: string_formats.AtUri  #: Uri.
    labels: t.Optional[t.List['models.ComAtprotoLabelDefs.Label']] = None  #: Labels.
    like_count: t.Optional[int] = Field(default=None, ge=0)  #: Like count.
    reason_types: t.Optional[t.List['models.ComAtprotoModerationDefs.ReasonType']] = (
        None  #: The set of report reason 'codes' which are in-scope for this service to review and action. These usually align to policy categories. If not defined (distinct from empty array), all reason types are allowed.
    )
    subject_collections: t.Optional[t.List[string_formats.Nsid]] = (
        None  #: Set of record types (collection NSIDs) which can be reported to this service. If not defined (distinct from empty array), default is any record type.
    )
    subject_types: t.Optional[t.List['models.ComAtprotoModerationDefs.SubjectType']] = (
        None  #: The set of subject types (account, record, etc) this service accepts reports on.
    )
    viewer: t.Optional['models.AppBskyLabelerDefs.LabelerViewerState'] = None  #: Viewer.

    py_type: t.Literal['app.bsky.labeler.defs#labelerViewDetailed'] = Field(
        default='app.bsky.labeler.defs#labelerViewDetailed', alias='$type', frozen=True
    )


class LabelerViewerState(base.ModelBase):
    """Definition model for :obj:`app.bsky.labeler.defs`."""

    like: t.Optional[string_formats.AtUri] = None  #: Like.

    py_type: t.Literal['app.bsky.labeler.defs#labelerViewerState'] = Field(
        default='app.bsky.labeler.defs#labelerViewerState', alias='$type', frozen=True
    )


class LabelerPolicies(base.ModelBase):
    """Definition model for :obj:`app.bsky.labeler.defs`."""

    label_values: t.List[
        'models.ComAtprotoLabelDefs.LabelValue'
    ]  #: The label values which this labeler publishes. May include global or custom labels.
    label_value_definitions: t.Optional[t.List['models.ComAtprotoLabelDefs.LabelValueDefinition']] = (
        None  #: Label values created by this labeler and scoped exclusively to it. Labels defined here will override global label definitions for this labeler.
    )

    py_type: t.Literal['app.bsky.labeler.defs#labelerPolicies'] = Field(
        default='app.bsky.labeler.defs#labelerPolicies', alias='$type', frozen=True
    )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`tools.ozone.setting.listOptions`."""

    cursor: t.Optional[str] = None  #: Cursor.
    keys: t.Optional[t.List[string_formats.Nsid]] = Field(
        default=None, max_length=100
    )  #: Filter for only the specified keys. Ignored if prefix is provided.
    limit: t.Optional[int] = Field(default=50, ge=1, le=100)  #: Limit.
    prefix: t.Optional[str] = None  #: Filter keys by prefix.
    scope: t.Optional[t.Union[t.Literal['instance'], t.Literal['personal'], str]] = 'instance'  #: Scope.


class ParamsDict(t.TypedDict):
    cursor: te.NotRequired[t.Optional[str]]  #: Cursor.
    keys: te.NotRequired[
        t.Optional[t.List[string_formats.Nsid]]
    ]  #: Filter for only the specified keys. Ignored if prefix is provided.
    limit: te.NotRequired[t.Optional[int]]  #: Limit.
    prefix: te.NotRequired[t.Optional[str]]  #: Filter keys by prefix.
    scope: te.NotRequired[t.Optional[t.Union[t.Literal['instance'], t.Literal['personal'], str]]]  #: Scope.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`tools.ozone.setting.listOptions`."""

    options: t.List['models.ToolsOzoneSettingDefs.Option']  #: Options.
    cursor: t.Optional[str] = None  #: Cursor.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from pydantic import Field

from atproto_client.models import base


class SigDetail(base.ModelBase):
    """Definition model for :obj:`tools.ozone.signature.defs`."""

    property: str  #: Property.
    value: str  #: Value.

    py_type: t.Literal['tools.ozone.signature.defs#sigDetail'] = Field(
        default='tools.ozone.signature.defs#sigDetail', alias='$type', frozen=True
    )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`tools.ozone.moderation.queryStatuses`."""

    appealed: t.Optional[bool] = None  #: Get subjects in unresolved appealed status.
    collections: t.Optional[t.List[string_formats.Nsid]] = Field(
        default=None, max_length=20
    )  #: If specified, subjects belonging to the given collections will be returned. When subjectType is set to 'account', this will be ignored.
    comment: t.Optional[str] = None  #: Search subjects by keyword from comments.
    cursor: t.Optional[str] = None  #: Cursor.
    exclude_tags: t.Optional[t.List[str]] = None  #: Exclude tags.
    hosting_deleted_after: t.Optional[string_formats.DateTime] = (
        None  #: Search subjects where the associated record/account was deleted after a given timestamp.
    )
    hosting_deleted_before: t.Optional[string_formats.DateTime] = (
        None  #: Search subjects where the associated record/account was deleted before a given timestamp.
    )
    hosting_statuses: t.Optional[t.List[str]] = None  #: Search subjects by the status of the associated record/account.
    hosting_updated_after: t.Optional[string_formats.DateTime] = (
        None  #: Search subjects where the associated record/account was updated after a given timestamp.
    )
    hosting_updated_before: t.Optional[string_formats.DateTime] = (
        None  #: Search subjects where the associated record/account was updated before a given timestamp.
    )
    ignore_subjects: t.Optional[t.List[string_formats.Uri]] = None  #: Ignore subjects.
    include_all_user_records: t.Optional[bool] = (
        None  #: All subjects, or subjects from given 'collections' param, belonging to the account specified in the 'subject' param will be returned.
    )
    include_muted: t.Optional[bool] = (
        None  #: By default, we don't include muted subjects in the results. Set this to true to include them.
    )
    last_reviewed_by: t.Optional[string_formats.Did] = (
        None  #: Get all subject statuses that were reviewed by a specific moderator.
    )
    limit: t.Optional[int] = Field(default=50, ge=1, le=100)  #: Limit.
    min_account_suspend_count: t.Optional[int] = (
        None  #: If specified, only subjects that belong to an account that has at least this many suspensions will be returned.
    )
    min_priority_score: t.Optional[int] = Field(
        default=None, ge=0, le=100
    )  #: If specified, only subjects that have priority score value above the given value will be returned.
    min_reported_records_count: t.Optional[int] = (
        None  #: If specified, only subjects that belong to an account that has at least this many reported records will be returned.
    )
    min_takendown_records_count: t.Optional[int] = (
        None  #: If specified, only subjects that belong to an account that has at least this many taken down records will be returned.
    )
    only_muted: t.Optional[bool] = None  #: When set to true, only muted subjects and reporters will be returned.
    queue_count: t.Optional[int] = (
        None  #: Number of queues being used by moderators. Subjects will be split among all queues.
    )
    queue_index: t.Optional[int] = (
        None  #: Index of the queue to fetch subjects from. Works only when queueCount value is specified.
    )
    queue_seed: t.Optional[str] = None  #: A seeder to shuffle/balance the queue items.
    reported_after: t.Optional[string_formats.DateTime] = None  #: Search subjects reported after a given timestamp.
    reported_before: t.Optional[string_formats.DateTime] = None  #: Search subjects reported before a given timestamp.
    review_state: t.Optional[str] = None  #: Specify when fetching subjects in a certain state.
    reviewed_after: t.Optional[string_formats.DateTime] = None  #: Search subjects reviewed after a given timestamp.
    reviewed_before: t.Optional[string_formats.DateTime] = None  #: Search subjects reviewed before a given timestamp.
    sort_direction: t.Optional[t.Union[t.Literal['asc'], t.Literal['desc']]] = 'desc'  #: Sort direction.
    sort_field: t.Optional[
        t.Union[
            t.Literal['lastReviewedAt'],
            t.Literal['lastReportedAt'],
            t.Literal['reportedRecordsCount'],
            t.Literal['takendownRecordsCount'],
            t.Literal['priorityScore'],
        ]
    ] = 'lastReportedAt'  #: Sort field.
    subject: t.Optional[string_formats.Uri] = None  #: The subject to get the status for.
    subject_type: t.Optional[t.Union[t.Literal['account'], t.Literal['record'], str]] = (
        None  #: If specified, subjects of the given type (account or record) will be returned. When this is set to 'account' the 'collections' parameter will be ignored. When includeAllUserRecords or subject is set, this will be ignored.
    )
    tags: t.Optional[t.List[str]] = Field(
        default=None, max_length=25
    )  #: Tags. Items in this array are applied with OR filters. To apply AND filter, put all tags in the same string and separate using && characters.
    takendown: t.Optional[bool] = None  #: Get subjects that were taken down.


class ParamsDict(t.TypedDict):
    appealed: te.NotRequired[t.Optional[bool]]  #: Get subjects in unresolved appealed status.
    collections: te.NotRequired[
        t.Optional[t.List[string_formats.Nsid]]
    ]  #: If specified, subjects belonging to the given collections will be returned. When subjectType is set to 'account', this will be ignored.
    comment: te.NotRequired[t.Optional[str]]  #: Search subjects by keyword from comments.
    cursor: te.NotRequired[t.Optional[str]]  #: Cursor.
    exclude_tags: te.NotRequired[t.Optional[t.List[str]]]  #: Exclude tags.
    hosting_deleted_after: te.NotRequired[
        t.Optional[string_formats.DateTime]
    ]  #: Search subjects where the associated record/account was deleted after a given timestamp.
    hosting_deleted_before: te.NotRequired[
        t.Optional[string_formats.DateTime]
    ]  #: Search subjects where the associated record/account was deleted before a given timestamp.
    hosting_statuses: te.NotRequired[
        t.Optional[t.List[str]]
    ]  #: Search subjects by the status of the associated record/account.
    hosting_updated_after: te.NotRequired[
        t.Optional[string_formats.DateTime]
    ]  #: Search subjects where the associated record/account was updated after a given timestamp.
    hosting_updated_before: te.NotRequired[
        t.Optional[string_formats.DateTime]
    ]  #: Search subjects where the associated record/account was updated before a given timestamp.
    ignore_subjects: te.NotRequired[t.Optional[t.List[string_formats.Uri]]]  #: Ignore subjects.
    include_all_user_records: te.NotRequired[
        t.Optional[bool]
    ]  #: All subjects, or subjects from given 'collections' param, belonging to the account specified in the 'subject' param will be returned.
    include_muted: te.NotRequired[
        t.Optional[bool]
    ]  #: By default, we don't include muted subjects in the results. Set this to true to include them.
    last_reviewed_by: te.NotRequired[
        t.Optional[string_formats.Did]
    ]  #: Get all subject statuses that were reviewed by a specific moderator.
    limit: te.NotRequired[t.Optional[int]]  #: Limit.
    min_account_suspend_count: te.NotRequired[
        t.Optional[int]
    ]  #: If specified, only subjects that belong to an account that has at least this many suspensions will be returned.
    min_priority_score: te.NotRequired[
        t.Optional[int]
    ]  #: If specified, only subjects that have priority score value above the given value will be returned.
    min_reported_records_count: te.NotRequired[
        t.Optional[int]
    ]  #: If specified, only subjects that belong to an account that has at least this many reported records will be returned.
    min_takendown_records_count: te.NotRequired[
        t.Optional[int]
    ]  #: If specified, only subjects that belong to an account that has at least this many taken down records will be returned.
    only_muted: te.NotRequired[
        t.Optional[bool]
    ]  #: When set to true, only muted subjects and reporters will be returned.
    queue_count: te.NotRequired[
        t.Optional[int]
    ]  #: Number of queues being used by moderators. Subjects will be split among all queues.
    queue_index: te.NotRequired[
        t.Optional[int]
    ]  #: Index of the queue to fetch subjects from. Works only when queueCount value is specified.
    queue_seed: te.NotRequired[t.Optional[str]]  #: A seeder to shuffle/balance the queue items.
    reported_after: te.NotRequired[
        t.Optional[string_formats.DateTime]
    ]  #: Search subjects reported after a given timestamp.
    reported_before: te.NotRequired[
        t.Optional[string_formats.DateTime]
    ]  #: Search subjects reported before a given timestamp.
    review_state: te.NotRequired[t.Optional[str]]  #: Specify when fetching subjects in a certain state.
    reviewed_after: te.NotRequired[
        t.Optional[string_formats.DateTime]
    ]  #: Search subjects reviewed after a given timestamp.
    reviewed_before: te.NotRequired[
        t.Optional[string_formats.DateTime]
    ]  #: Search subjects reviewed before a given timestamp.
    sort_direction: te.NotRequired[t.Optional[t.Union[t.Literal['asc'], t.Literal['desc']]]]  #: Sort direction.
    sort_field: te.NotRequired[
        t.Optional[
            t.Union[
                t.Literal['lastReviewedAt'],
                t.Literal['lastReportedAt'],
                t.Literal['reportedRecordsCount'],
                t.Literal['takendownRecordsCount'],
                t.Literal['priorityScore'],
            ]
        ]
    ]  #: Sort field.
    subject: te.NotRequired[t.Optional[string_formats.Uri]]  #: The subject to get the status for.
    subject_type: te.NotRequired[
        t.Optional[t.Union[t.Literal['account'], t.Literal['record'], str]]
    ]  #: If specified, subjects of the given type (account or record) will be returned. When this is set to 'account' the 'collections' parameter will be ignored. When includeAllUserRecords or subject is set, this will be ignored.
    tags: te.NotRequired[
        t.Optional[t.List[str]]
    ]  #: Tags. Items in this array are applied with OR filters. To apply AND filter, put all tags in the same string and separate using && characters.
    takendown: te.NotRequired[t.Optional[bool]]  #: Get subjects that were taken down.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`tools.ozone.moderation.queryStatuses`."""

    subject_statuses: t.List['models.ToolsOzoneModerationDefs.SubjectStatusView']  #: Subject statuses.
    cursor: t.Optional[str] = None  #: Cursor.



import typing as t

from atproto_client.client.base import ClientBase
from atproto_client.namespaces import sync_ns

# TODO(MarshalX): This file should be autogenerated!


class ClientRaw(ClientBase):
    """Group all root namespaces."""

    com: 'sync_ns.ComNamespace'
    app: 'sync_ns.AppNamespace'
    tools: 'sync_ns.ToolsNamespace'
    chat: 'sync_ns.ChatNamespace'

    def __init__(self, *args: t.Any, **kwargs: t.Any) -> None:
        super().__init__(*args, **kwargs)

        self.com = sync_ns.ComNamespace(self)
        self.app = sync_ns.AppNamespace(self)
        self.tools = sync_ns.ToolsNamespace(self)
        self.chat = sync_ns.ChatNamespace(self)



import re
import typing as t
from dataclasses import dataclass, field

from atproto_core.exceptions import InvalidNsidError

_NSID_DELIM = '.'
_NSID_NAMESPACE = '*'

_MAX_NSID_LEN = 253 + 1 + 63
_MAX_NSID_PART_LEN = 63
_MIN_NSID_PART_LEN = 1
_MIN_NSID_SEGMENTS_COUNT = 3

_NSID_VALID_CHARS_REGEX = r'^[a-zA-Z0-9.-]*$'
_NSID_FIRST_SEGMENT_NO_LEADING_DIGIT_REGEX = r'^[0-9]'
_NSID_LAST_SEGMENT_REGEX = r'^[a-zA-Z][a-zA-Z0-9]*$'

_NSID_PART_FORBIDDEN_START = '-'
_NSID_PART_FORBIDDEN_END = '-'

Segments = t.List[str]


@dataclass
class NSID:
    """NameSpaced IDs (NSIDs).

    Examples:
        com.example.status

        io.social.getFeed

        net.users.bob.ping
    """

    segments: Segments = field(default_factory=list)

    @classmethod
    def from_str(cls, nsid: str) -> 'NSID':
        """Create `NSID` instance from string."""
        validate_nsid(nsid)
        return cls(segments=get_nsid_segments(nsid))

    @property
    def authority(self) -> str:
        """Get authority of NSID.

        com.example.thing
        ^^^^^^^^^^^--------> example.com

        delim joined self.segments[:-1][::-1]
        """
        segments_without_name = self.segments[:-1]
        segments_without_name.reverse()

        return _NSID_DELIM.join(segments_without_name)

    @property
    def name(self) -> str:
        """Get name."""
        return self.segments[-1]

    def __str__(self) -> str:
        return _NSID_DELIM.join(self.segments)

    def __hash__(self) -> int:
        return hash(str(self))

    def __eq__(self, other: t.Any) -> bool:
        if isinstance(other, NSID):
            return hash(self) == hash(other)

        return False


def get_nsid_segments(nsid: str) -> Segments:
    return nsid.split('.')


def _exclude_nsid_namespace_if_exists(segments: Segments) -> str:
    last_segment = segments[-1]

    if last_segment == _NSID_NAMESPACE:
        segments = segments[:-1]

    return _NSID_DELIM.join(segments)


def _validate_nsid_segment(segment: str, is_first_segment: bool, is_last_segment: bool) -> bool:
    """Validate NSID segment.

    Args:
        segment: Segment to validate.
        is_first_segment: The first segment indicator.
        is_last_segment: The last segment indicator.

    Returns:
        :obj:`bool`: The validation result is always True.

    Raises:
        :class:`atproto.exceptions.InvalidNsidError`: Invalid NSID exception.
    """
    segment_len = len(segment)
    if segment_len < _MIN_NSID_PART_LEN:
        raise InvalidNsidError(f'NSID segment is too short. Min len: {_MIN_NSID_PART_LEN}')
    if segment_len > _MAX_NSID_PART_LEN:
        raise InvalidNsidError(f'NSID segment is too long. Max len: {_MAX_NSID_PART_LEN}')

    if segment.startswith(_NSID_PART_FORBIDDEN_START):
        raise InvalidNsidError(f'NSID segment cannot start with {_NSID_PART_FORBIDDEN_START}')
    if segment.endswith(_NSID_PART_FORBIDDEN_END):
        raise InvalidNsidError(f'NSID segment cannot end with {_NSID_PART_FORBIDDEN_END}')

    if is_first_segment and re.search(_NSID_FIRST_SEGMENT_NO_LEADING_DIGIT_REGEX, segment):
        raise InvalidNsidError('NSID first segment cannot start with a digit')

    if is_last_segment and not re.search(_NSID_LAST_SEGMENT_REGEX, segment):
        raise InvalidNsidError('NSID final segments must start with a letter and contain only letters and digits')

    return True


def _validate_nsid(nsid: str) -> bool:
    """Validate NSID.

    Args:
        nsid: NSID to validate.

    Returns:
        :obj:`bool`: Validation result is always True.

    Raises:
        :class:`atproto.exceptions.InvalidNsidError`: Invalid NSID exception.
    """
    if len(nsid) > _MAX_NSID_LEN:
        raise InvalidNsidError(f'NSID is too long. Max len: {_MAX_NSID_LEN}')
    if not re.search(_NSID_VALID_CHARS_REGEX, nsid):
        raise InvalidNsidError('NSID contains invalid chars. Only letters, digits, dots, and hyphens are allowed')

    segments = get_nsid_segments(nsid)
    if len(segments) < _MIN_NSID_SEGMENTS_COUNT:
        raise InvalidNsidError(f'NSID must contains at least {_MIN_NSID_SEGMENTS_COUNT} segments')

    segments_count = len(segments)
    for segment_index, segment in enumerate(segments):
        is_first_segment = segment_index == 0
        is_last_segment = segment_index + 1 == segments_count
        _validate_nsid_segment(segment, is_first_segment, is_last_segment)

    return True


def validate_nsid(nsid: str, *, soft_fail: bool = False) -> bool:
    """Validate NSID.

    Args:
        nsid: NSID to validate.
        soft_fail: enable to return False on fall instead of exception

    Returns:
        :obj:`bool`: Validation result.

    Raises:
        :class:`atproto.exceptions.InvalidNsidError`: Invalid NSID exception.
    """
    try:
        return _validate_nsid(nsid)
    except InvalidNsidError as e:
        if soft_fail:
            return False

        raise e



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te

from atproto_client.models import base, string_formats


class Data(base.DataModelBase):
    """Input data model for :obj:`chat.bsky.moderation.updateActorAccess`."""

    actor: string_formats.Did  #: Actor.
    allow_access: bool  #: Allow access.
    ref: t.Optional[str] = None  #: Ref.


class DataDict(t.TypedDict):
    actor: string_formats.Did  #: Actor.
    allow_access: bool  #: Allow access.
    ref: te.NotRequired[t.Optional[str]]  #: Ref.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
    from atproto_client.models.unknown_type import UnknownType
from atproto_client.models import base


class Main(base.ModelBase):
    """Definition model for :obj:`app.bsky.embed.record`."""

    record: 'models.ComAtprotoRepoStrongRef.Main'  #: Record.

    py_type: t.Literal['app.bsky.embed.record'] = Field(default='app.bsky.embed.record', alias='$type', frozen=True)


class View(base.ModelBase):
    """Definition model for :obj:`app.bsky.embed.record`."""

    record: te.Annotated[
        t.Union[
            'models.AppBskyEmbedRecord.ViewRecord',
            'models.AppBskyEmbedRecord.ViewNotFound',
            'models.AppBskyEmbedRecord.ViewBlocked',
            'models.AppBskyEmbedRecord.ViewDetached',
            'models.AppBskyFeedDefs.GeneratorView',
            'models.AppBskyGraphDefs.ListView',
            'models.AppBskyLabelerDefs.LabelerView',
            'models.AppBskyGraphDefs.StarterPackViewBasic',
        ],
        Field(discriminator='py_type'),
    ]  #: Record.

    py_type: t.Literal['app.bsky.embed.record#view'] = Field(
        default='app.bsky.embed.record#view', alias='$type', frozen=True
    )


class ViewRecord(base.ModelBase):
    """Definition model for :obj:`app.bsky.embed.record`."""

    author: 'models.AppBskyActorDefs.ProfileViewBasic'  #: Author.
    cid: string_formats.Cid  #: Cid.
    indexed_at: string_formats.DateTime  #: Indexed at.
    uri: string_formats.AtUri  #: Uri.
    value: 'UnknownType'  #: The record data itself.
    embeds: t.Optional[
        t.List[
            te.Annotated[
                t.Union[
                    'models.AppBskyEmbedImages.View',
                    'models.AppBskyEmbedVideo.View',
                    'models.AppBskyEmbedExternal.View',
                    'models.AppBskyEmbedRecord.View',
                    'models.AppBskyEmbedRecordWithMedia.View',
                ],
                Field(discriminator='py_type'),
            ]
        ]
    ] = None  #: Embeds.
    labels: t.Optional[t.List['models.ComAtprotoLabelDefs.Label']] = None  #: Labels.
    like_count: t.Optional[int] = None  #: Like count.
    quote_count: t.Optional[int] = None  #: Quote count.
    reply_count: t.Optional[int] = None  #: Reply count.
    repost_count: t.Optional[int] = None  #: Repost count.

    py_type: t.Literal['app.bsky.embed.record#viewRecord'] = Field(
        default='app.bsky.embed.record#viewRecord', alias='$type', frozen=True
    )


class ViewNotFound(base.ModelBase):
    """Definition model for :obj:`app.bsky.embed.record`."""

    not_found: bool = Field(frozen=True)  #: Not found.
    uri: string_formats.AtUri  #: Uri.

    py_type: t.Literal['app.bsky.embed.record#viewNotFound'] = Field(
        default='app.bsky.embed.record#viewNotFound', alias='$type', frozen=True
    )


class ViewBlocked(base.ModelBase):
    """Definition model for :obj:`app.bsky.embed.record`."""

    author: 'models.AppBskyFeedDefs.BlockedAuthor'  #: Author.
    blocked: bool = Field(frozen=True)  #: Blocked.
    uri: string_formats.AtUri  #: Uri.

    py_type: t.Literal['app.bsky.embed.record#viewBlocked'] = Field(
        default='app.bsky.embed.record#viewBlocked', alias='$type', frozen=True
    )


class ViewDetached(base.ModelBase):
    """Definition model for :obj:`app.bsky.embed.record`."""

    detached: bool = Field(frozen=True)  #: Detached.
    uri: string_formats.AtUri  #: Uri.

    py_type: t.Literal['app.bsky.embed.record#viewDetached'] = Field(
        default='app.bsky.embed.record#viewDetached', alias='$type', frozen=True
    )



import typing as t
from enum import Enum

import typing_extensions as te
from atproto_core.exceptions import AtProtocolError

_ATPROTO_PROXY_HEADER = 'atproto-proxy'
_ATPROTO_ACCEPT_LABELERS_HEADER = 'atproto-accept-labelers'


class HeadersConfigurationMethodsMixin:
    BSKY_CHAT_DID: t.ClassVar[t.Literal['did:web:api.bsky.chat']] = 'did:web:api.bsky.chat'

    # Bluesky hardcoded labeler https://docs.bsky.app/blog/blueskys-moderation-architecture
    BSKY_LABELER_DID: t.ClassVar[t.Literal['did:plc:ar7c4by46qjdydhdevvrndac']] = 'did:plc:ar7c4by46qjdydhdevvrndac'

    class AtprotoServiceType(Enum):
        """The type of atproto service."""

        ATPROTO_LABELER = 'atproto_labeler'
        BSKY_CHAT = 'bsky_chat'

    def clone(self) -> te.Self:
        """Clone the client instance.

        Used to customize atproto proxy and set of labeler services.

        Returns:
            Cloned client instance.
        """
        cloned_client = super().clone()

        # share the same objects to avoid conflicts with session changes
        cloned_client.me = self.me
        cloned_client._session = self._session
        cloned_client._session_dispatcher = self._session_dispatcher

        return cloned_client

    def with_proxy(self, service_type: t.Union[AtprotoServiceType, str], did: str) -> te.Self:
        """Get a new client instance with the atproto-proxy header configured.

        Args:
            service_type: The type of service.
            did: The DID of the proxy.

        Returns:
            :obj:`self`: Configured client instance.
        """
        cloned_client = self.clone()
        cloned_client.configure_proxy_header(service_type, did)
        return cloned_client

    def with_labelers(self, labeler_dids: t.List[str]) -> te.Self:
        """Get a new client instance with the atproto-accept-labelers header configured.

        Args:
            labeler_dids: The DIDs of the labelers.

        Returns:
            :obj:`self`: Configured client instance.
        """
        cloned_client = self.clone()
        cloned_client.configure_labelers_header(labeler_dids)
        return cloned_client

    def configure_proxy_header(self, service_type: t.Union[AtprotoServiceType, str], did: str) -> None:
        """Configure the atproto-proxy header to be applied on requests.

        Args:
            service_type: The type of service.
            did: The DID of the proxy.
        """
        if not did.startswith('did:'):
            raise AtProtocolError('Invalid DID format')

        if isinstance(service_type, self.AtprotoServiceType):
            service_type = service_type.value

        proxy_header = f'{did}#{service_type}'
        self.request.add_additional_header(_ATPROTO_PROXY_HEADER, proxy_header)

    def configure_labelers_header(self, labeler_dids: t.List[str]) -> None:
        """Configure the atproto-labelers header to be applied on requests.

        Args:
            labeler_dids: The DIDs of the labelers.
        """
        labelers_prepared = [f'{labeler_did};redact' for labeler_did in labeler_dids if labeler_did.startswith('did:')]
        labelers_header_value = ','.join(labelers_prepared)

        self.request.add_additional_header(_ATPROTO_ACCEPT_LABELERS_HEADER, labelers_header_value)

    def with_bsky_chat_proxy(self) -> te.Self:
        """Get a new client instance with the atproto-proxy header configured for bsky.chat.

        Returns:
            :obj:`self`: Configured client instance.
        """
        return self.with_proxy(self.AtprotoServiceType.BSKY_CHAT, self.BSKY_CHAT_DID)

    def with_bsky_labeler(self) -> te.Self:
        """Get a new client instance with the atproto-accept-labelers header configured for Bluesky Labeler.

        Returns:
            :obj:`self`: Configured client instance.
        """
        return self.with_labelers([self.BSKY_LABELER_DID])



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`tools.ozone.moderation.getSubjects`."""

    subjects: t.List[str] = Field(min_length=1, max_length=100)  #: Subjects.


class ParamsDict(t.TypedDict):
    subjects: t.List[str]  #: Subjects.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`tools.ozone.moderation.getSubjects`."""

    subjects: t.List['models.ToolsOzoneModerationDefs.SubjectView']  #: Subjects.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

HostStatus = t.Union[
    t.Literal['active'], t.Literal['idle'], t.Literal['offline'], t.Literal['throttled'], t.Literal['banned'], str
]  #: Host status
